@@ -9,6 +9,7 @@ config = NewConfig()
 config:Add(OptCCompiler("compiler"))
 config:Add(OptTestCompileC("stackprotector", "int main(){return 0;}", "-fstack-protector -fstack-protector-all"))
 config:Add(OptTestCompileC("minmacosxsdk", "int main(){return 0;}", "-mmacosx-version-min=10.7 -isysroot /Developer/SDKs/MacOSX10.7.sdk"))
+config:Add(OptTestCompileC("buildwithoutsseflag", "#include <immintrin.h>\nint main(){_mm_pause();return 0;}", ""))
 config:Add(OptLibrary("zlib", "zlib.h", false))
 config:Add(SDL.OptFind("sdl", true))
 config:Add(FreeType.OptFind("freetype", true))
@@ -19,11 +20,16 @@ generated_icon_dir = "build/icons"
 builddir = "build/%(arch)s/%(conf)s"
 content_src_dir = "datasrc/"
 
+python_in_path = ExecuteSilent("python -V") == 0
+
 -- data compiler
 function Python(name)
 	if family == "windows" then
-		-- Python is usually registered for .py files in Windows
-		return str_replace(name, "/", "\\")
+		name = str_replace(name, "/", "\\")
+		if not python_in_path then
+			-- Python is usually registered for .py files in Windows
+			return name
+		end
 	end
 	return "python " .. name
 end
@@ -120,7 +126,7 @@ function GenerateMacOSXSettings(settings, conf, arch, compiler)
 		os.exit(1)
 	end
 
-	-- c++ stdlib needed 
+	-- c++ stdlib needed
 	settings.cc.flags:Add("--stdlib=libc++")
 	settings.link.flags:Add("--stdlib=libc++")
 	-- this also needs the macOS min SDK version to be at least 10.7
@@ -164,12 +170,14 @@ function GenerateMacOSXSettings(settings, conf, arch, compiler)
 	BuildClient(settings)
 
 	-- Content
-	BuildContent(settings)
+	BuildContent(settings, arch, conf)
 end
 
 function GenerateLinuxSettings(settings, conf, arch, compiler)
 	if arch == "x86" then
-		settings.cc.flags:Add("-msse2") -- for the _mm_pause call
+		if config.buildwithoutsseflag.value == false then
+			settings.cc.flags:Add("-msse2") -- for the _mm_pause call
+		end
 		settings.cc.flags:Add("-m32")
 		settings.link.flags:Add("-m32")
 	elseif arch == "x86_64" then
@@ -200,11 +208,10 @@ function GenerateLinuxSettings(settings, conf, arch, compiler)
 	-- Client
 	settings.link.libs:Add("X11")
 	settings.link.libs:Add("GL")
-	settings.link.libs:Add("GLU")
 	BuildClient(settings)
 
 	-- Content
-	BuildContent(settings)
+	BuildContent(settings, arch, conf)
 end
 
 function GenerateSolarisSettings(settings, conf, arch, compiler)
@@ -234,6 +241,7 @@ function GenerateWindowsSettings(settings, conf, target_arch, compiler)
 	end
 
 	local icons = SharedIcons(compiler)
+	local manifests = SharedManifests(compiler)
 
 	-- Required libs
 	settings.link.libs:Add("gdi32")
@@ -261,13 +269,13 @@ function GenerateWindowsSettings(settings, conf, target_arch, compiler)
 
 	-- Client
 	settings.link.extrafiles:Add(icons.client)
+	settings.link.extrafiles:Add(manifests.client)
 	settings.link.libs:Add("opengl32")
-	settings.link.libs:Add("glu32")
 	settings.link.libs:Add("winmm")
 	BuildClient(settings)
 
 	-- Content
-	BuildContent(settings)
+	BuildContent(settings, target_arch, conf)
 end
 
 function SharedCommonFiles()
@@ -321,6 +329,14 @@ function SharedIcons(compiler)
 	return shared_icons[compiler]
 end
 
+function SharedManifests(compiler)
+	if not shared_manifests then
+		local client_manifest = ResCompile("other/manifest/teeworlds.rc", compiler)
+		shared_manifests = {client=client_manifest}
+	end
+	return shared_manifests
+end
+
 function BuildEngineCommon(settings)
 	settings.link.extrafiles:Merge(Compile(settings, Collect("src/engine/shared/*.cpp", "src/base/*.c")))
 end
@@ -367,9 +383,27 @@ function BuildVersionserver(settings)
 	return Link(settings, "versionsrv", Compile(settings, Collect("src/versionsrv/*.cpp")), libs["zlib"], libs["md5"])
 end
 
-function BuildContent(settings)
+function BuildContent(settings, arch, conf)
 	local content = {}
 	table.insert(content, CopyToDir(settings.link.Output(settings, "data"), CollectRecursive(content_src_dir .. "*.png", content_src_dir .. "*.wv", content_src_dir .. "*.ttf", content_src_dir .. "*.txt", content_src_dir .. "*.map", content_src_dir .. "*.rules", content_src_dir .. "*.json")))
+	if family == "windows" then
+		if arch == "x86_64" then
+			_arch = "64"
+		else
+			_arch = "32"
+		end
+		-- dependencies
+		dl = Python("scripts/download.py")
+		AddJob({
+				"other/freetype/include/ft2build.h", "other/freetype/windows/lib" .. _arch .. "/freetype.dll",
+				"other/sdl/include/SDL.h", "other/sdl/windows/lib" .. _arch .. "/SDL2.dll"
+			}, "Downloading freetype and SDL2", dl .. " freetype sdl"
+		)
+		table.insert(content, CopyFile(settings.link.Output(settings, "") .. "/SDL2.dll", "other/sdl/windows/lib" .. _arch .. "/SDL2.dll"))
+		table.insert(content, CopyFile(settings.link.Output(settings, "") .. "/freetype.dll", "other/freetype/windows/lib" .. _arch .. "/freetype.dll"))
+		AddDependency(settings.link.Output(settings, "") .. "/SDL2.dll", "other/sdl/include/SDL.h")
+		AddDependency(settings.link.Output(settings, "") .. "/freetype.dll", "other/freetype/include/ft2build.h")
+	end
 	PseudoTarget(settings.link.Output(settings, "content") .. settings.link.extension, content)
 end
 
@@ -414,6 +448,8 @@ function GenerateSettings(conf, arch, builddir, compiler)
 	end
 	
 	settings.cc.includes:Add("src")
+	settings.cc.includes:Add("src/engine/external/pnglite")
+	settings.cc.includes:Add("src/engine/external/wavpack")
 	settings.cc.includes:Add(generated_src_dir)
 	
 	if family == "windows" then
@@ -431,7 +467,7 @@ function GenerateSettings(conf, arch, builddir, compiler)
 	return settings
 end
 
--- String formatting wth named parameters, by RiciLake http://lua-users.org/wiki/StringInterpolation
+-- String formatting with named parameters, by RiciLake http://lua-users.org/wiki/StringInterpolation
 function interp(s, tab)
 	return (s:gsub('%%%((%a%w*)%)([-0-9%.]*[cdeEfgGiouxXsq])',
 			function(k, fmt)
@@ -499,6 +535,7 @@ for a, cur_arch in ipairs(archs) do
 		end
 	end
 end
+
 for cur_name, cur_target in pairs(targets) do
 	-- Supertarget for all configurations and architectures of that target
 	PseudoTarget(cur_name, subtargets[cur_target])
