@@ -1,4 +1,3 @@
-import copy
 from datatypes import *
 
 class Sound(Struct):
@@ -235,19 +234,21 @@ image_null = Image("null", "")
 image_particles = Image("particles", "particles.png")
 image_game = Image("game", "game.png")
 image_browseicons = Image("browseicons", "ui/icons/browse.png", 1)
+image_browsericon = Image("browser", "ui/icons/browser.png", 1)
 image_emoticons = Image("emoticons", "emoticons.png")
 image_demobuttons = Image("demobuttons", "ui/demo_buttons.png", 1)
 image_fileicons = Image("fileicons", "ui/file_icons.png", 1)
 image_guibuttons = Image("guibuttons", "ui/gui_buttons.png", 1)
 image_guiicons = Image("guiicons", "ui/gui_icons.png", 1)
 image_menuicons = Image("menuicons", "ui/icons/menu.png", 1)
+image_soundicons = Image("soundicons", "ui/sound_icons.png", 1)
 image_toolicons = Image("toolicons", "ui/icons/tools.png", 1)
 image_arrowicons = Image("arrowicons", "ui/icons/arrows.png", 1)
 image_friendicons = Image("friendicons", "ui/icons/friend.png", 1)
-image_infoicons = Image("infoicons", "ui/icons/info.png", 1)
 image_levelicons = Image("levelicons", "ui/icons/level.png", 1)
 image_sidebaricons = Image("sidebaricons", "ui/icons/sidebar.png", 1)
 image_chatwhisper = Image("chatwhisper", "ui/icons/chat_whisper.png", 1)
+image_timerclock = Image("timerclock", "ui/icons/timer_clock.png", 1)
 
 container.images.Add(image_null)
 container.images.Add(image_game)
@@ -257,6 +258,7 @@ container.images.Add(Image("cursor", "ui/gui_cursor.png"))
 container.images.Add(Image("banner", "ui/gui_logo.png"))
 container.images.Add(image_emoticons)
 container.images.Add(image_browseicons)
+container.images.Add(image_browsericon)
 container.images.Add(Image("console_bg", "ui/console.png"))
 container.images.Add(Image("console_bar", "ui/console_bar.png"))
 container.images.Add(image_demobuttons)
@@ -265,13 +267,15 @@ container.images.Add(image_guibuttons)
 container.images.Add(image_guiicons)
 container.images.Add(Image("no_skinpart", "ui/no_skinpart.png"))
 container.images.Add(image_menuicons)
+container.images.Add(image_soundicons)
 container.images.Add(image_toolicons)
 container.images.Add(image_arrowicons)
 container.images.Add(image_friendicons)
-container.images.Add(image_infoicons)
 container.images.Add(image_levelicons)
 container.images.Add(image_sidebaricons)
 container.images.Add(image_chatwhisper)
+container.images.Add(Image("raceflag", "race_flag.png"))
+container.images.Add(image_timerclock)
 
 container.pickups.Add(Pickup("health"))
 container.pickups.Add(Pickup("armor"))
@@ -279,6 +283,8 @@ container.pickups.Add(Pickup("grenade"))
 container.pickups.Add(Pickup("shotgun"))
 container.pickups.Add(Pickup("laser"))
 container.pickups.Add(Pickup("ninja", 90, 90))
+container.pickups.Add(Pickup("gun"))
+container.pickups.Add(Pickup("hammer"))
 
 set_particles = SpriteSet("particles", image_particles, 8, 8)
 set_game = SpriteSet("game", image_game, 32, 16)
@@ -289,19 +295,22 @@ set_tee_hands = SpriteSet("tee_hands", image_null, 2, 1)
 set_tee_feet = SpriteSet("tee_feet", image_null, 2, 1)
 set_tee_eyes = SpriteSet("tee_eyes", image_null, 2, 4)
 set_tee_hats = SpriteSet("tee_hats", image_null, 1, 4)
+set_tee_bot = SpriteSet("tee_bot", image_null, 12, 5)
 set_browseicons = SpriteSet("browseicons", image_browseicons, 4, 2)
+set_browsericon = SpriteSet("browsericon", image_browsericon, 1, 2)
 set_emoticons = SpriteSet("emoticons", image_emoticons, 4, 4)
 set_demobuttons = SpriteSet("demobuttons", image_demobuttons, 5, 1)
 set_fileicons = SpriteSet("fileicons", image_fileicons, 8, 1)
 set_guibuttons = SpriteSet("guibuttons", image_guibuttons, 12, 4)
 set_guiicons = SpriteSet("guiicons", image_guiicons, 8, 2)
-set_menuicons = SpriteSet("menuicons", image_menuicons, 2, 2)
+set_menuicons = SpriteSet("menuicons", image_menuicons, 4, 4)
 set_toolicons = SpriteSet("toolicons", image_toolicons, 4, 2)
+set_soundicons = SpriteSet("guiicons", image_soundicons, 1, 2)
 set_arrowicons = SpriteSet("arrowicons", image_arrowicons, 4, 3)
 set_friendicons = SpriteSet("friendicons", image_friendicons, 2, 2)
-set_infoicons = SpriteSet("infoicons", image_infoicons, 1, 2)
 set_levelicons = SpriteSet("levelicons", image_levelicons, 4, 4)
 set_sidebaricons = SpriteSet("sidebaricons", image_sidebaricons, 4, 2)
+set_timerclock = SpriteSet("timerclock", image_timerclock, 1, 2)
 
 container.spritesets.Add(set_particles)
 container.spritesets.Add(set_game)
@@ -312,6 +321,7 @@ container.spritesets.Add(set_tee_hands)
 container.spritesets.Add(set_tee_feet)
 container.spritesets.Add(set_tee_eyes)
 container.spritesets.Add(set_tee_hats)
+container.spritesets.Add(set_tee_bot)
 container.spritesets.Add(set_browseicons)
 container.spritesets.Add(set_emoticons)
 container.spritesets.Add(set_demobuttons)
@@ -319,12 +329,14 @@ container.spritesets.Add(set_fileicons)
 container.spritesets.Add(set_guibuttons)
 container.spritesets.Add(set_guiicons)
 container.spritesets.Add(set_menuicons)
+container.spritesets.Add(set_soundicons)
 container.spritesets.Add(set_toolicons)
 container.spritesets.Add(set_arrowicons)
 container.spritesets.Add(set_friendicons)
-container.spritesets.Add(set_infoicons)
 container.spritesets.Add(set_levelicons)
 container.spritesets.Add(set_sidebaricons)
+container.spritesets.Add(set_timerclock)
+container.spritesets.Add(set_browsericon)
 
 
 container.sprites.Add(Sprite("part_slice", set_particles, 0,0,1,1))
@@ -401,6 +413,8 @@ container.sprites.Add(Sprite("pickup_grenade", set_game, 2,8,7,2))
 container.sprites.Add(Sprite("pickup_shotgun", set_game, 2,6,8,2))
 container.sprites.Add(Sprite("pickup_laser", set_game, 2,12,7,3))
 container.sprites.Add(Sprite("pickup_ninja", set_game, 2,10,8,2))
+container.sprites.Add(Sprite("pickup_gun", set_game, 2,4,4,2))
+container.sprites.Add(Sprite("pickup_hammer", set_game, 2,1,4,3))
 
 container.sprites.Add(Sprite("flag_blue", set_game, 12,8,4,8))
 container.sprites.Add(Sprite("flag_red", set_game, 16,8,4,8))
@@ -437,6 +451,10 @@ container.sprites.Add(Sprite("tee_hats_top2", set_tee_hats, 0,1,1,1))
 container.sprites.Add(Sprite("tee_hats_side1", set_tee_hats, 0,2,1,1))
 container.sprites.Add(Sprite("tee_hats_side2", set_tee_hats, 0,3,1,1))
 
+container.sprites.Add(Sprite("tee_bot_glow", set_tee_bot, 0,0,4,4))
+container.sprites.Add(Sprite("tee_bot_foreground", set_tee_bot, 4,0,4,4))
+container.sprites.Add(Sprite("tee_bot_background", set_tee_bot, 8,0,4,4))
+
 container.sprites.Add(Sprite("oop", set_emoticons, 0, 0, 1, 1))
 container.sprites.Add(Sprite("exclamation", set_emoticons, 1, 0, 1, 1))
 container.sprites.Add(Sprite("hearts", set_emoticons, 2, 0, 1, 1))
@@ -456,16 +474,12 @@ container.sprites.Add(Sprite("question", set_emoticons, 3, 3, 1, 1))
 
 container.sprites.Add(Sprite("browse_lock_a", set_browseicons, 0,0,1,1))
 container.sprites.Add(Sprite("browse_lock_b", set_browseicons, 0,1,1,1))
-#container.sprites.Add(Sprite("browse_lock_c", set_browseicons, 2,0,1,1))
 container.sprites.Add(Sprite("browse_unpure_a", set_browseicons, 1,0,1,1))
 container.sprites.Add(Sprite("browse_unpure_b", set_browseicons, 1,1,1,1))
-#container.sprites.Add(Sprite("browse_unpure_c", set_browseicons, 2,1,1,1))
 container.sprites.Add(Sprite("browse_star_a", set_browseicons, 2,0,1,1))
 container.sprites.Add(Sprite("browse_star_b", set_browseicons, 2,1,1,1))
-#container.sprites.Add(Sprite("browse_star_c", set_browseicons, 2,2,1,1))
 container.sprites.Add(Sprite("browse_heart_a", set_browseicons, 3,0,1,1))
 container.sprites.Add(Sprite("browse_heart_b", set_browseicons, 3,1,1,1))
-#container.sprites.Add(Sprite("browse_heart_c", set_browseicons, 2,3,1,1))
 
 container.sprites.Add(Sprite("demobutton_play", set_demobuttons, 0,0,1,1))
 container.sprites.Add(Sprite("demobutton_pause", set_demobuttons, 1,0,1,1))
@@ -488,9 +502,13 @@ container.sprites.Add(Sprite("guiicon_friend", set_guiicons, 4,0,4,2))
 
 container.sprites.Add(Sprite("menu_checkbox_active", set_menuicons, 0,0,1,1))
 container.sprites.Add(Sprite("menu_checkbox_inactive", set_menuicons, 0,1,1,1))
+container.sprites.Add(Sprite("menu_checkbox_hover", set_menuicons, 0,2,1,1))
 container.sprites.Add(Sprite("menu_collapsed", set_menuicons, 1,0,1,1))
 container.sprites.Add(Sprite("menu_expanded", set_menuicons, 1,1,1,1))
 
+container.sprites.Add(Sprite("soundicon_on", set_soundicons, 0,0,1,1))
+container.sprites.Add(Sprite("soundicon_mute", set_soundicons, 0,1,1,1))
+
 container.sprites.Add(Sprite("tool_up_a", set_toolicons, 0,0,1,1))
 container.sprites.Add(Sprite("tool_up_b", set_toolicons, 0,1,1,1))
 container.sprites.Add(Sprite("tool_down_a", set_toolicons, 1,0,1,1))
@@ -518,9 +536,6 @@ container.sprites.Add(Sprite("friend_plus_b", set_friendicons, 0,1,1,1))
 container.sprites.Add(Sprite("friend_x_a", set_friendicons, 1,0,1,1))
 container.sprites.Add(Sprite("friend_x_b", set_friendicons, 1,1,1,1))
 
-container.sprites.Add(Sprite("info_a", set_infoicons, 0,0,1,1))
-container.sprites.Add(Sprite("info_b", set_infoicons, 0,1,1,1))
-
 container.sprites.Add(Sprite("level_a_on", set_levelicons, 0,0,1,1))
 container.sprites.Add(Sprite("level_a_a", set_levelicons, 0,1,1,1))
 container.sprites.Add(Sprite("level_a_b", set_levelicons, 0,2,1,1))
@@ -540,6 +555,12 @@ container.sprites.Add(Sprite("sidebar_filter_b", set_sidebaricons, 2,1,1,1))
 container.sprites.Add(Sprite("sidebar_info_a", set_sidebaricons, 3,0,1,1))
 container.sprites.Add(Sprite("sidebar_info_b", set_sidebaricons, 3,1,1,1))
 
+container.sprites.Add(Sprite("browser_a", set_browsericon, 0,0,1,1))
+container.sprites.Add(Sprite("browser_b", set_browsericon, 0,1,1,1))
+
+container.sprites.Add(Sprite("timerclock_a", set_timerclock, 0,0,1,1))
+container.sprites.Add(Sprite("timerclock_b", set_timerclock, 0,1,1,1))
+
 anim = Animation("base")
 anim.body.frames.Add(AnimKeyframe(0, 0, -4, 0))
 anim.back_foot.frames.Add(AnimKeyframe(0, 0, 10, 0))
