@@ -16,10 +16,11 @@ JSON_KEY_AUTHORS="authors"
 JSON_KEY_TRANSL="translated strings"
 JSON_KEY_UNTRANSL="needs translation"
 JSON_KEY_OLDTRANSL="old translations"
+JSON_KEY_CTXT="context"
 JSON_KEY_OR="or"
 JSON_KEY_TR="tr"
 
-SOURCE_LOCALIZE_RE=re.compile(br'Localize\("(?P<str>([^"\\]|\\.)*)"(, "(?P<ctxt>([^"\\]|\\.)*)")?\)')
+SOURCE_LOCALIZE_RE=re.compile(br'Localize\("(?P<str>([^"\\]|\\.)*)"(, ?"(?P<ctxt>([^"\\]|\\.)*)")?\)')
 
 def parse_source():
 	l10n = defaultdict(lambda: [])
@@ -75,8 +76,10 @@ def write_languagefile(outputfilename, l10n_src, old_l10n_data):
 		JSON_KEY_UNTRANSL,
 		JSON_KEY_TRANSL,
 	):
+		if type_ not in old_l10n_data:
+			continue
 		translations.update({
-			t[JSON_KEY_OR]: t[JSON_KEY_TR]
+			(t[JSON_KEY_OR], t.get(JSON_KEY_CTXT)): t[JSON_KEY_TR]
 			for t in old_l10n_data[type_]
 			if t[JSON_KEY_TR]
 		})
@@ -86,10 +89,11 @@ def write_languagefile(outputfilename, l10n_src, old_l10n_data):
 	old_items = set(translations) - set(l10n_src)
 	new_items = set(l10n_src) - set(translations)
 
-	for msg in all_items:
+	for msg, ctxt in all_items:
 		po.append(polib.POEntry(
 			msgid=msg,
-			msgstr=translations.get(msg, ""),
+			msgctxt=ctxt,
+			msgstr=translations.get((msg, ctxt), ""),
 			obsolete=(msg in old_items),
 			occurrences=l10n_src[msg],
 		))
@@ -114,13 +118,13 @@ if __name__ == '__main__':
 		if(commenttxt):
 			commenttxt = 'Context: '+commenttxt
 		po.append(polib.POEntry(msgid=msg, msgstr="", occurrences=occurrences, msgctxt=ctxt, comment=commenttxt))
-	po.save('data/languages/base.pot')
+	po.save('datasrc/languages/base.pot')
 
-	for filename in os.listdir("data/languages"):
+	for filename in os.listdir("datasrc/languages"):
 		try:
 			if (os.path.splitext(filename)[1] == ".json"
 					and filename != "index.json"):
-				filename = "data/languages/" + filename
+				filename = "datasrc/languages/" + filename
 				write_languagefile(filename, l10n_src, load_languagefile(filename))
 		except Exception as e:
 			print("Failed on {0}, re-raising for traceback".format(filename))
