@@ -6,7 +6,8 @@
 #include <base/tl/threading.h>
 
 #include <base/system.h>
-#include <engine/external/pnglite/pnglite.h>
+
+#include <pnglite.h>
 
 #include <engine/shared/config.h>
 #include <engine/graphics.h>
@@ -41,7 +42,7 @@ void CGraphics_Threaded::FlushVertices()
 	int NumVerts = m_NumVertices;
 	m_NumVertices = 0;
 
-	CCommandBuffer::SCommand_Render Cmd;
+	CCommandBuffer::CRenderCommand Cmd;
 	Cmd.m_State = m_State;
 
 	if(m_Drawing == DRAWING_QUADS)
@@ -57,13 +58,13 @@ void CGraphics_Threaded::FlushVertices()
 	else
 		return;
 
-	Cmd.m_pVertices = (CCommandBuffer::SVertex *)m_pCommandBuffer->AllocData(sizeof(CCommandBuffer::SVertex)*NumVerts);
+	Cmd.m_pVertices = (CCommandBuffer::CVertex *)m_pCommandBuffer->AllocData(sizeof(CCommandBuffer::CVertex)*NumVerts);
 	if(Cmd.m_pVertices == 0x0)
 	{
 		// kick command buffer and try again
 		KickCommandBuffer();
 
-		Cmd.m_pVertices = (CCommandBuffer::SVertex *)m_pCommandBuffer->AllocData(sizeof(CCommandBuffer::SVertex)*NumVerts);
+		Cmd.m_pVertices = (CCommandBuffer::CVertex *)m_pCommandBuffer->AllocData(sizeof(CCommandBuffer::CVertex)*NumVerts);
 		if(Cmd.m_pVertices == 0x0)
 		{
 			dbg_msg("graphics", "failed to allocate data for vertices");
@@ -76,8 +77,8 @@ void CGraphics_Threaded::FlushVertices()
 	{
 		// kick command buffer and try again
 		KickCommandBuffer();
-		
-		Cmd.m_pVertices = (CCommandBuffer::SVertex *)m_pCommandBuffer->AllocData(sizeof(CCommandBuffer::SVertex)*NumVerts);
+
+		Cmd.m_pVertices = (CCommandBuffer::CVertex *)m_pCommandBuffer->AllocData(sizeof(CCommandBuffer::CVertex)*NumVerts);
 		if(Cmd.m_pVertices == 0x0)
 		{
 			dbg_msg("graphics", "failed to allocate data for vertices");
@@ -91,7 +92,7 @@ void CGraphics_Threaded::FlushVertices()
 		}
 	}
 
-	mem_copy(Cmd.m_pVertices, m_aVertices, sizeof(CCommandBuffer::SVertex)*NumVerts);
+	mem_copy(Cmd.m_pVertices, m_aVertices, sizeof(CCommandBuffer::CVertex)*NumVerts);
 }
 
 void CGraphics_Threaded::AddVertices(int Count)
@@ -101,7 +102,7 @@ void CGraphics_Threaded::AddVertices(int Count)
 		FlushVertices();
 }
 
-void CGraphics_Threaded::Rotate4(const CCommandBuffer::SPoint &rCenter, CCommandBuffer::SVertex *pPoints)
+void CGraphics_Threaded::Rotate4(const CCommandBuffer::CPoint &rCenter, CCommandBuffer::CVertex *pPoints)
 {
 	float c = cosf(m_Rotation);
 	float s = sinf(m_Rotation);
@@ -272,7 +273,7 @@ int CGraphics_Threaded::UnloadTexture(CTextureHandle *Index)
 	if(!Index->IsValid())
 		return 0;
 
-	CCommandBuffer::SCommand_Texture_Destroy Cmd;
+	CCommandBuffer::CTextureDestroyCommand Cmd;
 	Cmd.m_Slot = Index->Id();
 	m_pCommandBuffer->AddCommand(Cmd);
 
@@ -304,7 +305,10 @@ static int ImageFormatToPixelSize(int Format)
 
 int CGraphics_Threaded::LoadTextureRawSub(CTextureHandle TextureID, int x, int y, int Width, int Height, int Format, const void *pData)
 {
-	CCommandBuffer::SCommand_Texture_Update Cmd;
+	if(!TextureID.IsValid())
+		return 0;
+
+	CCommandBuffer::CTextureUpdateCommand Cmd;
 	Cmd.m_Slot = TextureID.Id();
 	Cmd.m_X = x;
 	Cmd.m_Y = y;
@@ -328,7 +332,7 @@ int CGraphics_Threaded::LoadTextureRawSub(CTextureHandle TextureID, int x, int y
 IGraphics::CTextureHandle CGraphics_Threaded::LoadTextureRaw(int Width, int Height, int Format, const void *pData, int StoreFormat, int Flags)
 {
 	// don't waste memory on texture if we are stress testing
-	if(g_Config.m_DbgStress)
+	if(m_pConfig->m_DbgStress)
 		return m_InvalidTexture;
 
 	// grab texture
@@ -336,7 +340,7 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTextureRaw(int Width, int Heig
 	m_FirstFreeTexture = m_aTextureIndices[Tex];
 	m_aTextureIndices[Tex] = -1;
 
-	CCommandBuffer::SCommand_Texture_Create Cmd;
+	CCommandBuffer::CTextureCreateCommand Cmd;
 	Cmd.m_Slot = Tex;
 	Cmd.m_Width = Width;
 	Cmd.m_Height = Height;
@@ -349,9 +353,9 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTextureRaw(int Width, int Heig
 	Cmd.m_Flags = CCommandBuffer::TEXFLAG_TEXTURE2D;
 	if(Flags&IGraphics::TEXLOAD_NOMIPMAPS)
 		Cmd.m_Flags |= CCommandBuffer::TEXFLAG_NOMIPMAPS;
-	if(g_Config.m_GfxTextureCompression)
+	if(m_pConfig->m_GfxTextureCompression)
 		Cmd.m_Flags |= CCommandBuffer::TEXFLAG_COMPRESSED;
-	if(g_Config.m_GfxTextureQuality || Flags&TEXLOAD_NORESAMPLE)
+	if(m_pConfig->m_GfxTextureQuality || Flags&TEXLOAD_NORESAMPLE)
 		Cmd.m_Flags |= CCommandBuffer::TEXFLAG_QUALITY;
 	if(Flags&IGraphics::TEXLOAD_ARRAY_256)
 	{
@@ -363,13 +367,13 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTextureRaw(int Width, int Heig
 	if(Flags&IGraphics::TEXLOAD_LINEARMIPMAPS)
 		Cmd.m_Flags |= CCommandBuffer::TEXTFLAG_LINEARMIPMAPS;
 
-	
+
 	// copy texture data
 	int MemSize = Width*Height*Cmd.m_PixelSize;
 	void *pTmpData = mem_alloc(MemSize, sizeof(void*));
 	mem_copy(pTmpData, pData, MemSize);
 	Cmd.m_pData = pTmpData;
-	
+
 
 	//
 	m_pCommandBuffer->AddCommand(Cmd);
@@ -393,7 +397,7 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTexture(const char *pFilename,
 
 		ID = LoadTextureRaw(Img.m_Width, Img.m_Height, Img.m_Format, Img.m_pData, StoreFormat, Flags);
 		mem_free(Img.m_pData);
-		if(ID.Id() != m_InvalidTexture.Id() && g_Config.m_Debug)
+		if(ID.Id() != m_InvalidTexture.Id() && m_pConfig->m_Debug)
 			dbg_msg("graphics/texture", "loaded %s", pFilename);
 		return ID;
 	}
@@ -403,7 +407,7 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTexture(const char *pFilename,
 
 int CGraphics_Threaded::LoadPNG(CImageInfo *pImg, const char *pFilename, int StorageType)
 {
-	char aCompleteFilename[512];
+	char aCompleteFilename[IO_MAX_PATH_LENGTH];
 	unsigned char *pBuffer;
 	png_t Png; // ignore_convention
 
@@ -465,7 +469,7 @@ void CGraphics_Threaded::ScreenshotDirect(const char *pFilename)
 	CImageInfo Image;
 	mem_zero(&Image, sizeof(Image));
 
-	CCommandBuffer::SCommand_Screenshot Cmd;
+	CCommandBuffer::CScreenshotCommand Cmd;
 	Cmd.m_pImage = &Image;
 	Cmd.m_X = 0; Cmd.m_Y = 0;
 	Cmd.m_W = -1; Cmd.m_H = -1;
@@ -506,7 +510,7 @@ void CGraphics_Threaded::TextureSet(CTextureHandle TextureID)
 
 void CGraphics_Threaded::Clear(float r, float g, float b)
 {
-	CCommandBuffer::SCommand_Clear Cmd;
+	CCommandBuffer::CClearCommand Cmd;
 	Cmd.m_Color.r = r;
 	Cmd.m_Color.g = g;
 	Cmd.m_Color.b = b;
@@ -613,7 +617,7 @@ void CGraphics_Threaded::QuadsSetSubsetFree(
 	// tileset fallback system
 	if(m_pBackend->GetTextureArraySize() > 1 && TextureIndex >= 0)
 		TilesetFallbackSystem(TextureIndex);
-	
+
 	m_State.m_TextureArrayIndex = m_TextureArrayIndex;
 
 	m_aTexture[0].u = x0; m_aTexture[0].v = y0;
@@ -638,8 +642,7 @@ void CGraphics_Threaded::QuadsDraw(CQuadItem *pArray, int Num)
 
 void CGraphics_Threaded::QuadsDrawTL(const CQuadItem *pArray, int Num)
 {
-	CCommandBuffer::SPoint Center;
-	Center.z = 0;
+	CCommandBuffer::CPoint Center;
 
 	dbg_assert(m_Drawing == DRAWING_QUADS, "called Graphics()->QuadsDrawTL without begin");
 
@@ -739,12 +742,16 @@ void CGraphics_Threaded::QuadsText(float x, float y, float Size, const char *pTe
 int CGraphics_Threaded::IssueInit()
 {
 	int Flags = 0;
-	if(g_Config.m_GfxBorderless) Flags |= IGraphicsBackend::INITFLAG_BORDERLESS;
-	if(g_Config.m_GfxFullscreen) Flags |= IGraphicsBackend::INITFLAG_FULLSCREEN;
-	if(g_Config.m_GfxVsync) Flags |= IGraphicsBackend::INITFLAG_VSYNC;
-	if(g_Config.m_DbgResizable) Flags |= IGraphicsBackend::INITFLAG_RESIZABLE;
+	if(m_pConfig->m_GfxBorderless) Flags |= IGraphicsBackend::INITFLAG_BORDERLESS;
+	if(m_pConfig->m_GfxFullscreen) Flags |= IGraphicsBackend::INITFLAG_FULLSCREEN;
+	if(m_pConfig->m_GfxVsync) Flags |= IGraphicsBackend::INITFLAG_VSYNC;
+	if(m_pConfig->m_GfxHighdpi) Flags |= IGraphicsBackend::INITFLAG_HIGHDPI;
+	if(m_pConfig->m_DbgResizable) Flags |= IGraphicsBackend::INITFLAG_RESIZABLE;
+	if(m_pConfig->m_GfxUseX11XRandRWM) Flags |= IGraphicsBackend::INITFLAG_X11XRANDR;
 
-	return m_pBackend->Init("Teeworlds", &g_Config.m_GfxScreen, &g_Config.m_GfxScreenWidth, &g_Config.m_GfxScreenHeight, g_Config.m_GfxFsaaSamples, Flags, &m_DesktopScreenWidth, &m_DesktopScreenHeight);
+	return m_pBackend->Init("Teeworlds", &m_pConfig->m_GfxScreen, &m_pConfig->m_GfxScreenWidth,
+			&m_pConfig->m_GfxScreenHeight, &m_ScreenWidth, &m_ScreenHeight, m_pConfig->m_GfxFsaaSamples,
+			Flags, &m_DesktopScreenWidth, &m_DesktopScreenHeight);
 }
 
 int CGraphics_Threaded::InitWindow()
@@ -753,12 +760,12 @@ int CGraphics_Threaded::InitWindow()
 		return 0;
 
 	// try disabling fsaa
-	while(g_Config.m_GfxFsaaSamples)
+	while(m_pConfig->m_GfxFsaaSamples)
 	{
-		g_Config.m_GfxFsaaSamples--;
+		m_pConfig->m_GfxFsaaSamples--;
 
-		if(g_Config.m_GfxFsaaSamples)
-			dbg_msg("gfx", "lowering FSAA to %d and trying again", g_Config.m_GfxFsaaSamples);
+		if(m_pConfig->m_GfxFsaaSamples)
+			dbg_msg("gfx", "lowering FSAA to %d and trying again", m_pConfig->m_GfxFsaaSamples);
 		else
 			dbg_msg("gfx", "disabling FSAA and trying again");
 
@@ -767,11 +774,11 @@ int CGraphics_Threaded::InitWindow()
 	}
 
 	// try lowering the resolution
-	if(g_Config.m_GfxScreenWidth != 640 || g_Config.m_GfxScreenHeight != 480)
+	if(m_pConfig->m_GfxScreenWidth != 640 || m_pConfig->m_GfxScreenHeight != 480)
 	{
 		dbg_msg("gfx", "setting resolution to 640x480 and trying again");
-		g_Config.m_GfxScreenWidth = 640;
-		g_Config.m_GfxScreenHeight = 480;
+		m_pConfig->m_GfxScreenWidth = 640;
+		m_pConfig->m_GfxScreenHeight = 480;
 
 		if(IssueInit() == 0)
 			return 0;
@@ -786,12 +793,9 @@ int CGraphics_Threaded::Init()
 {
 	// fetch pointers
 	m_pStorage = Kernel()->RequestInterface<IStorage>();
+	m_pConfig = Kernel()->RequestInterface<IConfigManager>()->Values();
 	m_pConsole = Kernel()->RequestInterface<IConsole>();
 
-	// Set all z to -5.0f
-	for(int i = 0; i < MAX_VERTICES; i++)
-		m_aVertices[i].m_Pos.z = -5.0f;
-
 	// init textures
 	m_FirstFreeTexture = 0;
 	for(int i = 0; i < MAX_TEXTURES-1; i++)
@@ -802,10 +806,6 @@ int CGraphics_Threaded::Init()
 	if(InitWindow() != 0)
 		return -1;
 
-	// fetch final resolusion
-	m_ScreenWidth = g_Config.m_GfxScreenWidth;
-	m_ScreenHeight = g_Config.m_GfxScreenHeight;
-
 	// create command buffers
 	for(int i = 0; i < NUM_CMDBUFFERS; i++)
 		m_apCommandBuffers[i] = new CCommandBuffer(128*1024, 2*1024*1024);
@@ -892,7 +892,13 @@ void CGraphics_Threaded::SetWindowBordered(bool State)
 
 bool CGraphics_Threaded::SetWindowScreen(int Index)
 {
-	return m_pBackend->SetWindowScreen(Index);
+	if(m_pBackend->SetWindowScreen(Index))
+	{
+		// update resolution info
+		m_pBackend->GetDesktopResolution(Index, &m_DesktopScreenWidth, &m_DesktopScreenHeight);
+		return true;
+	}
+	return false;
 }
 
 int CGraphics_Threaded::GetWindowScreen()
@@ -920,7 +926,7 @@ void CGraphics_Threaded::ReadBackbuffer(unsigned char **ppPixels, int x, int y,
 	CImageInfo Image;
 	mem_zero(&Image, sizeof(Image));
 
-	CCommandBuffer::SCommand_Screenshot Cmd;
+	CCommandBuffer::CScreenshotCommand Cmd;
 	Cmd.m_pImage = &Image;
 	Cmd.m_X = x; Cmd.m_Y = y;
 	Cmd.m_W = w; Cmd.m_H = h;
@@ -953,8 +959,8 @@ void CGraphics_Threaded::Swap()
 	}
 
 	// add swap command
-	CCommandBuffer::SCommand_Swap Cmd;
-	Cmd.m_Finish = g_Config.m_GfxFinish;
+	CCommandBuffer::CSwapCommand Cmd;
+	Cmd.m_Finish = m_pConfig->m_GfxFinish;
 	m_pCommandBuffer->AddCommand(Cmd);
 
 	// kick the command buffer
@@ -965,7 +971,7 @@ bool CGraphics_Threaded::SetVSync(bool State)
 {
 	// add vsnc command
 	bool RetOk = 0;
-	CCommandBuffer::SCommand_VSync Cmd;
+	CCommandBuffer::CVSyncCommand Cmd;
 	Cmd.m_VSync = State ? 1 : 0;
 	Cmd.m_pRetOk = &RetOk;
 	m_pCommandBuffer->AddCommand(Cmd);
@@ -979,7 +985,7 @@ bool CGraphics_Threaded::SetVSync(bool State)
 // syncronization
 void CGraphics_Threaded::InsertSignal(semaphore *pSemaphore)
 {
-	CCommandBuffer::SCommand_Signal Cmd;
+	CCommandBuffer::CSignalCommand Cmd;
 	Cmd.m_pSemaphore = pSemaphore;
 	m_pCommandBuffer->AddCommand(Cmd);
 }
@@ -996,7 +1002,7 @@ void CGraphics_Threaded::WaitForIdle()
 
 int CGraphics_Threaded::GetVideoModes(CVideoMode *pModes, int MaxModes, int Screen)
 {
-	if(g_Config.m_GfxDisplayAllModes)
+	if(m_pConfig->m_GfxDisplayAllModes)
 	{
 		int Count = sizeof(g_aFakeModes)/sizeof(CVideoMode);
 		mem_copy(pModes, g_aFakeModes, sizeof(g_aFakeModes));
@@ -1010,7 +1016,7 @@ int CGraphics_Threaded::GetVideoModes(CVideoMode *pModes, int MaxModes, int Scre
 	mem_zero(&Image, sizeof(Image));
 
 	int NumModes = 0;
-	CCommandBuffer::SCommand_VideoModes Cmd;
+	CCommandBuffer::CVideoModesCommand Cmd;
 	Cmd.m_pModes = pModes;
 	Cmd.m_MaxModes = MaxModes;
 	Cmd.m_pNumModes = &NumModes;
