@@ -24,6 +24,7 @@ public:
 	void Update(bool ForceResort);	
 
 	// interface functions
+	int GetType() { return m_ActServerlistType; };
 	void SetType(int Type);
 	void Refresh(int RefreshFlags);
 	bool IsRefreshing() const { return m_pFirstReqServer != 0; }
@@ -40,12 +41,14 @@ public:
 	const CServerInfo *SortedGet(int FilterIndex, int Index) const { return &m_aServerlist[m_ActServerlistType].m_ppServerlist[m_ServerBrowserFilter.GetIndex(FilterIndex, Index)]->m_Info; };
 	const void *GetID(int FilterIndex, int Index) const { return m_ServerBrowserFilter.GetID(FilterIndex, Index); };
 
-	bool IsFavorite(const NETADDR &Addr) { return m_ServerBrowserFavorites.FindFavoriteByAddr(Addr, 0) != 0; }
-	void AddFavorite(const CServerInfo *pEntry);
-	void RemoveFavorite(const CServerInfo *pEntry);
+	void AddFavorite(const CServerInfo *pInfo);
+	void RemoveFavorite(const CServerInfo *pInfo);
+	void UpdateFavoriteState(CServerInfo *pInfo);
+	void SetFavoritePassword(const char *pAddress, const char *pPassword);
+	const char *GetFavoritePassword(const char *pAddress);
 
 	int AddFilter(const CServerFilterInfo *pFilterInfo) { return m_ServerBrowserFilter.AddFilter(pFilterInfo); };
-	void SetFilter(int Index, const CServerFilterInfo *pFilterInfo) { m_ServerBrowserFilter.SetFilter(Index, pFilterInfo); }; 
+	void SetFilter(int Index, const CServerFilterInfo *pFilterInfo) { m_ServerBrowserFilter.SetFilter(Index, pFilterInfo); };
 	void GetFilter(int Index, CServerFilterInfo *pFilterInfo) { m_ServerBrowserFilter.GetFilter(Index, pFilterInfo); };
 	void RemoveFilter(int Index) { m_ServerBrowserFilter.RemoveFilter(Index); };
 
@@ -56,6 +59,7 @@ public:
 
 private:
 	class CNetClient *m_pNetClient;
+	class CConfig *m_pConfig;
 	class IConsole *m_pConsole;
 	class IStorage *m_pStorage;
 	class IMasterServer *m_pMasterServer;
@@ -63,6 +67,7 @@ private:
 	class CServerBrowserFavorites m_ServerBrowserFavorites;
 	class CServerBrowserFilter m_ServerBrowserFilter;
 
+	class CConfig *Config() const { return m_pConfig; }
 	class IConsole *Console() const { return m_pConsole; }
 	class IStorage *Storage() const { return m_pStorage; }
 
