@@ -10,6 +10,7 @@ class CSound : public IEngineSound
 	int m_SoundEnabled;
 
 public:
+	CConfig *m_pConfig;
 	IEngineGraphics *m_pGraphics;
 	IStorage *m_pStorage;
 
@@ -21,10 +22,6 @@ public:
 
 	static void RateConvert(int SampleID);
 
-	// TODO: Refactor: clean this mess up
-	static IOHANDLE ms_File;
-	static int ReadData(void *pBuffer, int Size);
-
 	virtual bool IsSoundEnabled() { return m_SoundEnabled != 0; }
 
 	virtual CSampleHandle LoadWV(const char *pFilename);
