@@ -3,6 +3,8 @@
 #ifndef ENGINE_SHARED_MAPCHECKER_H
 #define ENGINE_SHARED_MAPCHECKER_H
 
+#include <base/hash.h>
+
 #include "memheap.h"
 
 class CMapChecker
@@ -15,6 +17,7 @@ class CMapChecker
 	struct CWhitelistEntry
 	{
 		char m_aMapName[MAX_MAP_LENGTH];
+		SHA256_DIGEST m_MapSha256;
 		unsigned m_MapCrc;
 		unsigned m_MapSize;
 		CWhitelistEntry *m_pNext;
@@ -31,7 +34,7 @@ class CMapChecker
 public:
 	CMapChecker();
 	void AddMaplist(struct CMapVersion *pMaplist, int Num);
-	bool IsMapValid(const char *pMapName, unsigned MapCrc, unsigned MapSize);
+	bool IsMapValid(const char *pMapName, const SHA256_DIGEST *pMapSha256, unsigned MapCrc, unsigned MapSize);
 	bool ReadAndValidateMap(class IStorage *pStorage, const char *pFilename, int StorageType);
 };
 
