@@ -1,5 +1,6 @@
 /* (c) Magnus Auvinen. See licence.txt in the root of the distribution for more information. */
 /* If you are missing that file, acquire a complete release at teeworlds.com.                */
+#include <base/math.h>
 #include <base/system.h>
 
 #include <engine/console.h>
@@ -8,75 +9,66 @@
 #include "network.h"
 
 
-bool CNetServer::Open(NETADDR BindAddr, CNetBan *pNetBan, int MaxClients, int MaxClientsPerIP, int Flags)
+bool CNetServer::Open(NETADDR BindAddr, CConfig *pConfig, IConsole *pConsole, IEngine *pEngine, CNetBan *pNetBan,
+	int MaxClients, int MaxClientsPerIP, NETFUNC_NEWCLIENT pfnNewClient, NETFUNC_DELCLIENT pfnDelClient, void *pUser)
 {
 	// zero out the whole structure
 	mem_zero(this, sizeof(*this));
 
 	// open socket
-	m_Socket = net_udp_create(BindAddr, 0);
-	if(!m_Socket.type)
+	NETSOCKET Socket = net_udp_create(BindAddr, 0);
+	if(!Socket.type)
 		return false;
 
-	m_TokenManager.Init(m_Socket);
-	m_TokenCache.Init(m_Socket, &m_TokenManager);
-
+	// init
 	m_pNetBan = pNetBan;
+	Init(Socket, pConfig, pConsole, pEngine);
 
-	// clamp clients
-	m_MaxClients = MaxClients;
-	if(m_MaxClients > NET_MAX_CLIENTS)
-		m_MaxClients = NET_MAX_CLIENTS;
-	if(m_MaxClients < 1)
-		m_MaxClients = 1;
+	m_TokenManager.Init(this);
+	m_TokenCache.Init(this, &m_TokenManager);
 
-	m_MaxClientsPerIP = MaxClientsPerIP;
+	m_NumClients = 0;
+	SetMaxClients(MaxClients);
+	SetMaxClientsPerIP(MaxClientsPerIP);
 
 	for(int i = 0; i < NET_MAX_CLIENTS; i++)
-		m_aSlots[i].m_Connection.Init(m_Socket, true);
+		m_aSlots[i].m_Connection.Init(this, true);
 
-	m_Flags = Flags;
+	m_pfnNewClient = pfnNewClient;
+	m_pfnDelClient = pfnDelClient;
+	m_UserPtr = pUser;
 
 	return true;
 }
 
-int CNetServer::SetCallbacks(NETFUNC_NEWCLIENT pfnNewClient, NETFUNC_DELCLIENT pfnDelClient, void *pUser)
+void CNetServer::Close()
 {
-	m_pfnNewClient = pfnNewClient;
-	m_pfnDelClient = pfnDelClient;
-	m_UserPtr = pUser;
-	return 0;
+	for(int i = 0; i < NET_MAX_CLIENTS; i++)
+		Drop(i, "Server shutdown");
+
+	Shutdown();
 }
 
-int CNetServer::Close()
+void CNetServer::Drop(int ClientID, const char *pReason)
 {
-	// TODO: implement me
-	return 0;
-}
+	if(ClientID < 0 || ClientID >= NET_MAX_CLIENTS || m_aSlots[ClientID].m_Connection.State() == NET_CONNSTATE_OFFLINE)
+		return;
 
-int CNetServer::Drop(int ClientID, const char *pReason)
-{
-	// TODO: insert lots of checks here
-	/*NETADDR Addr = ClientAddr(ClientID);
-
-	dbg_msg("net_server", "client dropped. cid=%d ip=%d.%d.%d.%d reason=\"%s\"",
-		ClientID,
-		Addr.ip[0], Addr.ip[1], Addr.ip[2], Addr.ip[3],
-		pReason
-		);*/
 	if(m_pfnDelClient)
 		m_pfnDelClient(ClientID, pReason, m_UserPtr);
 
 	m_aSlots[ClientID].m_Connection.Disconnect(pReason);
-
-	return 0;
+	m_NumClients--;
 }
 
 int CNetServer::Update()
 {
 	int64 Now = time_get();
-	for(int i = 0; i < MaxClients(); i++)
+	for(int i = 0; i < NET_MAX_CLIENTS; i++)
 	{
+		if(m_aSlots[i].m_Connection.State() == NET_CONNSTATE_OFFLINE)
+			continue;
+
 		m_aSlots[i].m_Connection.Update();
 		if(m_aSlots[i].m_Connection.State() == NET_CONNSTATE_ERROR)
 		{
@@ -103,34 +95,40 @@ int CNetServer::Recv(CNetChunk *pChunk, TOKEN *pResponseToken)
 {
 	while(1)
 	{
-		NETADDR Addr;
-
 		// check for a chunk
-		if(m_RecvUnpacker.FetchChunk(pChunk))
+		if(m_RecvUnpacker.IsActive() && m_RecvUnpacker.FetchChunk(pChunk))
 			return 1;
 
 		// TODO: empty the recvinfo
-		int Bytes = net_udp_recv(m_Socket, &Addr, m_RecvUnpacker.m_aBuffer, NET_MAX_PACKETSIZE);
-
+		NETADDR Addr;
+		int Result = UnpackPacket(&Addr, m_RecvUnpacker.m_aBuffer, &m_RecvUnpacker.m_Data);
 		// no more packets for now
-		if(Bytes <= 0)
+		if(Result > 0)
 			break;
 
-		if(CNetBase::UnpackPacket(m_RecvUnpacker.m_aBuffer, Bytes, &m_RecvUnpacker.m_Data) == 0)
+		if(!Result)
 		{
 			// check for bans
 			char aBuf[128];
-			if(NetBan() && NetBan()->IsBanned(&Addr, aBuf, sizeof(aBuf)))
+			int LastInfoQuery;
+			if(NetBan() && NetBan()->IsBanned(&Addr, aBuf, sizeof(aBuf), &LastInfoQuery))
 			{
-				// banned, reply with a message
-				CNetBase::SendControlMsg(m_Socket, &Addr, m_RecvUnpacker.m_Data.m_ResponseToken, 0, NET_CTRLMSG_CLOSE, aBuf, str_length(aBuf)+1);
+				// banned, reply with a message (5 second cooldown)
+				int Time = time_timestamp();
+				if(LastInfoQuery + 5 < Time)
+				{
+					SendControlMsg(&Addr, m_RecvUnpacker.m_Data.m_ResponseToken, 0, NET_CTRLMSG_CLOSE, aBuf, str_length(aBuf) + 1);
+				}
 				continue;
 			}
 
 			bool Found = false;
 			// try to find matching slot
-			for(int i = 0; i < MaxClients(); i++)
+			for(int i = 0; i < NET_MAX_CLIENTS; i++)
 			{
+				if(m_aSlots[i].m_Connection.State() == NET_CONNSTATE_OFFLINE)
+					continue;
+
 				if(net_addr_comp(m_aSlots[i].m_Connection.PeerAddress(), &Addr) == 0)
 				{
 					if(m_aSlots[i].m_Connection.Feed(&m_RecvUnpacker.m_Data, &Addr))
@@ -167,13 +165,21 @@ int CNetServer::Recv(CNetChunk *pChunk, TOKEN *pResponseToken)
 			{
 				if(m_RecvUnpacker.m_Data.m_aChunkData[0] == NET_CTRLMSG_CONNECT)
 				{
-					bool Found = false;
+					// check if there are free slots
+					if(m_NumClients >= m_MaxClients)
+					{
+						const char FullMsg[] = "This server is full";
+						SendControlMsg(&Addr, m_RecvUnpacker.m_Data.m_ResponseToken, 0, NET_CTRLMSG_CLOSE, FullMsg, sizeof(FullMsg));
+						continue;
+					}
 
 					// only allow a specific number of players with the same ip
 					NETADDR ThisAddr = Addr, OtherAddr;
 					int FoundAddr = 1;
 					ThisAddr.port = 0;
-					for(int i = 0; i < MaxClients(); i++)
+
+					bool Continue = false;
+					for(int i = 0; i < NET_MAX_CLIENTS; i++)
 					{
 						if(m_aSlots[i].m_Connection.State() == NET_CONNSTATE_OFFLINE)
 							continue;
@@ -186,17 +192,21 @@ int CNetServer::Recv(CNetChunk *pChunk, TOKEN *pResponseToken)
 							{
 								char aBuf[128];
 								str_format(aBuf, sizeof(aBuf), "Only %d players with the same IP are allowed", m_MaxClientsPerIP);
-								CNetBase::SendControlMsg(m_Socket, &Addr, m_RecvUnpacker.m_Data.m_ResponseToken, 0, NET_CTRLMSG_CLOSE, aBuf, str_length(aBuf) + 1);
-								return 0;
+								SendControlMsg(&Addr, m_RecvUnpacker.m_Data.m_ResponseToken, 0, NET_CTRLMSG_CLOSE, aBuf, str_length(aBuf) + 1);
+								Continue = true;
+								break;
 							}
 						}
 					}
 
-					for(int i = 0; i < MaxClients(); i++)
+					if(Continue)
+						continue;
+
+					for(int i = 0; i < NET_MAX_CLIENTS; i++)
 					{
 						if(m_aSlots[i].m_Connection.State() == NET_CONNSTATE_OFFLINE)
 						{
-							Found = true;
+							m_NumClients++;
 							m_aSlots[i].m_Connection.SetToken(m_RecvUnpacker.m_Data.m_Token);
 							m_aSlots[i].m_Connection.Feed(&m_RecvUnpacker.m_Data, &Addr);
 							if(m_pfnNewClient)
@@ -204,12 +214,6 @@ int CNetServer::Recv(CNetChunk *pChunk, TOKEN *pResponseToken)
 							break;
 						}
 					}
-
-					if(!Found)
-					{
-						const char FullMsg[] = "This server is full";
-						CNetBase::SendControlMsg(m_Socket, &Addr, m_RecvUnpacker.m_Data.m_ResponseToken, 0, NET_CTRLMSG_CLOSE, FullMsg, sizeof(FullMsg));
-					}
 				}
 				else if(m_RecvUnpacker.m_Data.m_aChunkData[0] == NET_CTRLMSG_TOKEN)
 					m_TokenCache.AddToken(&Addr, m_RecvUnpacker.m_Data.m_ResponseToken, NET_TOKENFLAG_RESPONSEONLY);
@@ -241,17 +245,24 @@ int CNetServer::Send(CNetChunk *pChunk, TOKEN Token)
 		}
 
 		if(pChunk->m_ClientID == -1)
-			for(int i = 0; i < MaxClients(); i++)
+		{
+			for(int i = 0; i < NET_MAX_CLIENTS; i++)
+			{
+				if(m_aSlots[i].m_Connection.State() == NET_CONNSTATE_OFFLINE)
+					continue;
+
 				if(net_addr_comp(&pChunk->m_Address, m_aSlots[i].m_Connection.PeerAddress()) == 0)
 				{
 					// upgrade the packet, now that we know its recipent
 					pChunk->m_ClientID = i;
 					break;
 				}
+			}
+		}
 
 		if(Token != NET_TOKEN_NONE)
 		{
-			CNetBase::SendPacketConnless(m_Socket, &pChunk->m_Address, Token, m_TokenManager.GenerateToken(&pChunk->m_Address), pChunk->m_pData, pChunk->m_DataSize);
+			SendPacketConnless(&pChunk->m_Address, Token, m_TokenManager.GenerateToken(&pChunk->m_Address), pChunk->m_pData, pChunk->m_DataSize);
 		}
 		else
 		{
@@ -262,7 +273,8 @@ int CNetServer::Send(CNetChunk *pChunk, TOKEN Token)
 			else
 			{
 				dbg_assert(pChunk->m_ClientID >= 0, "errornous client id");
-				dbg_assert(pChunk->m_ClientID < MaxClients(), "errornous client id");
+				dbg_assert(pChunk->m_ClientID < NET_MAX_CLIENTS, "errornous client id");
+				dbg_assert(m_aSlots[pChunk->m_ClientID].m_Connection.State() != NET_CONNSTATE_OFFLINE, "errornous client id");
 
 				m_aSlots[pChunk->m_ClientID].m_Connection.SendPacketConnless((const char *)pChunk->m_pData, pChunk->m_DataSize);
 			}
@@ -278,7 +290,8 @@ int CNetServer::Send(CNetChunk *pChunk, TOKEN Token)
 
 		int Flags = 0;
 		dbg_assert(pChunk->m_ClientID >= 0, "errornous client id");
-		dbg_assert(pChunk->m_ClientID < MaxClients(), "errornous client id");
+		dbg_assert(pChunk->m_ClientID < NET_MAX_CLIENTS, "errornous client id");
+		dbg_assert(m_aSlots[pChunk->m_ClientID].m_Connection.State() != NET_CONNSTATE_OFFLINE, "errornous client id");
 
 		if(pChunk->m_Flags&NETSENDFLAG_VITAL)
 			Flags = NET_CHUNKFLAG_VITAL;
@@ -296,13 +309,12 @@ int CNetServer::Send(CNetChunk *pChunk, TOKEN Token)
 	return 0;
 }
 
-void CNetServer::SetMaxClientsPerIP(int Max)
+void CNetServer::SetMaxClients(int MaxClients)
 {
-	// clamp
-	if(Max < 1)
-		Max = 1;
-	else if(Max > NET_MAX_CLIENTS)
-		Max = NET_MAX_CLIENTS;
+	m_MaxClients = clamp(MaxClients, 1, int(NET_MAX_CLIENTS));
+}
 
-	m_MaxClientsPerIP = Max;
+void CNetServer::SetMaxClientsPerIP(int MaxClientsPerIP)
+{
+	m_MaxClientsPerIP = clamp(MaxClientsPerIP, 1, int(NET_MAX_CLIENTS));
 }
