@@ -41,10 +41,10 @@ inline bool IsCharWhitespace(char c)
 
 void CBroadcast::RenderServerBroadcast()
 {
-	if(!g_Config.m_ClShowServerBroadcast || m_pClient->m_MuteServerBroadcast)
+	if(!Config()->m_ClShowServerBroadcast || m_pClient->m_MuteServerBroadcast)
 		return;
 
-	const bool ColoredBroadcastEnabled = g_Config.m_ClColoredBroadcast;
+	const bool ColoredBroadcastEnabled = Config()->m_ClColoredBroadcast;
 	const float Height = 300;
 	const float Width = Height*Graphics()->ScreenAspect();
 
@@ -259,7 +259,7 @@ void CBroadcast::OnReset()
 void CBroadcast::OnMessage(int MsgType, void* pRawMsg)
 {
 	// process server broadcast message
-	if(MsgType == NETMSGTYPE_SV_BROADCAST && g_Config.m_ClShowServerBroadcast &&
+	if(MsgType == NETMSGTYPE_SV_BROADCAST && Config()->m_ClShowServerBroadcast &&
 	   !m_pClient->m_MuteServerBroadcast)
 	{
 		CNetMsg_Sv_Broadcast *pMsg = (CNetMsg_Sv_Broadcast *)pRawMsg;
@@ -422,11 +422,14 @@ void CBroadcast::OnMessage(int MsgType, void* pRawMsg)
 
 void CBroadcast::OnRender()
 {
+	if(Client()->State() < IClient::STATE_ONLINE)
+		return;
+
 	// server broadcast
 	RenderServerBroadcast();
 
 	// client broadcast
-	if(m_pClient->m_pScoreboard->Active() || m_pClient->m_pMotd->IsActive())
+	if(m_pClient->m_pScoreboard->IsActive() || m_pClient->m_pMotd->IsActive())
 		return;
 
 	Graphics()->MapScreen(0, 0, 300*Graphics()->ScreenAspect(), 300);
