@@ -16,9 +16,10 @@ public:
 		bool m_Blocked;
 		IGraphics::CTextureHandle m_Texture;
 
-		bool operator<(const CCountryFlag &Other) { return str_comp(m_aCountryCodeString, Other.m_aCountryCodeString) < 0; }
+		bool operator<(const CCountryFlag &Other) const { return str_comp(m_aCountryCodeString, Other.m_aCountryCodeString) < 0; }
 	};
 
+	int GetInitAmount() const;
 	void OnInit();
 
 	int Num() const;
