@@ -29,7 +29,7 @@ void CEmoticon::ConEmote(IConsole::IResult *pResult, void *pUserData)
 void CEmoticon::OnConsoleInit()
 {
 	Console()->Register("+emote", "", CFGFLAG_CLIENT, ConKeyEmoticon, this, "Open emote selector");
-	Console()->Register("emote", "i", CFGFLAG_CLIENT, ConEmote, this, "Use emote");
+	Console()->Register("emote", "i[emote-id]", CFGFLAG_CLIENT, ConEmote, this, "Use emote");
 }
 
 void CEmoticon::OnReset()
@@ -93,6 +93,13 @@ void CEmoticon::DrawCircle(float x, float y, float r, int Segments)
 
 void CEmoticon::OnRender()
 {
+	if(Client()->State() < IClient::STATE_ONLINE)
+	{
+		m_Active = false;
+		m_WasActive = false;
+		return;
+	}
+
 	if(!m_Active)
 	{
 		if(m_WasActive && m_SelectedEmote != -1)
@@ -101,7 +108,7 @@ void CEmoticon::OnRender()
 		return;
 	}
 
-	if(m_pClient->m_Snap.m_SpecInfo.m_Active)
+	if(m_pClient->m_Snap.m_SpecInfo.m_Active || (m_pClient->m_Snap.m_pGameData && m_pClient->m_Snap.m_pGameData->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER))
 	{
 		m_Active = false;
 		m_WasActive = false;
