@@ -8,7 +8,8 @@
 
 #include <engine/graphics.h>
 #include <engine/demo.h>
-#include <engine/friends.h>
+#include <engine/contacts.h>
+#include <engine/serverbrowser.h>
 
 #include <game/voting.h>
 #include <game/client/component.h>
@@ -30,6 +31,56 @@ public:
 	virtual bool OnInput(IInput::CEvent Event);
 };
 
+class IScrollbarScale
+{
+public:
+	virtual float ToRelative(int AbsoluteValue, int Min, int Max) = 0;
+	virtual int ToAbsolute(float RelativeValue, int Min, int Max) = 0;
+};
+static class CLinearScrollbarScale : public IScrollbarScale
+{
+public:
+	float ToRelative(int AbsoluteValue, int Min, int Max)
+	{
+		return (AbsoluteValue - Min) / (float)(Max - Min);
+	}
+	int ToAbsolute(float RelativeValue, int Min, int Max)
+	{
+		return round_to_int(RelativeValue*(Max - Min) + Min + 0.1f);
+	}
+} LinearScrollbarScale;
+static class CLogarithmicScrollbarScale : public IScrollbarScale
+{
+private:
+	int m_MinAdjustment;
+public:
+	CLogarithmicScrollbarScale(int MinAdjustment)
+	{
+		m_MinAdjustment = max(MinAdjustment, 1); // must be at least 1 to support Min == 0 with logarithm
+	}
+	float ToRelative(int AbsoluteValue, int Min, int Max)
+	{
+		if(Min < m_MinAdjustment)
+		{
+			AbsoluteValue += m_MinAdjustment;
+			Min += m_MinAdjustment;
+			Max += m_MinAdjustment;
+		}
+		return (log(AbsoluteValue) - log(Min)) / (float)(log(Max) - log(Min));
+	}
+	int ToAbsolute(float RelativeValue, int Min, int Max)
+	{
+		int ResultAdjustment = 0;
+		if(Min < m_MinAdjustment)
+		{
+			Min += m_MinAdjustment;
+			Max += m_MinAdjustment;
+			ResultAdjustment = -m_MinAdjustment;
+		}
+		return round_to_int(exp(RelativeValue*(log(Max) - log(Min)) + log(Min))) + ResultAdjustment;
+	}
+} LogarithmicScrollbarScale(25);
+
 class CMenus : public CComponent
 {
 public:
@@ -42,7 +93,7 @@ public:
 	};
 
 private:
-	typedef float (*FDropdownCallback)(CUIRect View, void *pUser);
+	typedef float (CMenus::*FDropdownCallback)(CUIRect View);
 
 	float ButtonFade(CButtonContainer *pBC, float Seconds, int Checked=0);
 
@@ -55,86 +106,183 @@ private:
 	int DoButton_Menu(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect, const char *pImageName=0, int Corners=CUI::CORNER_ALL, float r=5.0f, float FontFactor=0.0f, vec4 ColorHot=vec4(1.0f, 1.0f, 1.0f, 0.75f), bool TextFade=true);
 	int DoButton_MenuTab(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Corners);
 	int DoButton_MenuTabTop(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect, float Alpha=1.0f, float FontAlpha=1.0f, int Corners=CUI::CORNER_ALL, float r=5.0f, float FontFactor=0.0f);
-	void DoButton_MenuTabTop_Dummy(const char *pText, int Checked, const CUIRect *pRect, float Alpha);
 	int DoButton_Customize(CButtonContainer *pBC, IGraphics::CTextureHandle Texture, int SpriteID, const CUIRect *pRect, float ImageRatio);
 
-	int DoButton_CheckBox_Common(const void *pID, const char *pText, const char *pBoxText, const CUIRect *pRect, bool Checked=false);
-	int DoButton_CheckBox(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
-	int DoButton_CheckBox_Number(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
+	int DoButton_CheckBox_Common(const void *pID, const char *pText, const char *pBoxText, const CUIRect *pRect, bool Checked=false, bool Locked=false);
+	int DoButton_CheckBox(const void *pID, const char *pText, int Checked, const CUIRect *pRect, bool Locked=false);
+	int DoButton_CheckBox_Number(const void *pID, const char *pText, int SelectedNumber, const CUIRect *pRect);
 
 	int DoButton_MouseOver(int ImageID, int SpriteID, const CUIRect *pRect);
 
-	/*static void ui_draw_menu_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-	static void ui_draw_keyselect_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-	static void ui_draw_menu_tab_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-	static void ui_draw_settings_tab_button(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-	*/
-
 	int DoIcon(int ImageId, int SpriteId, const CUIRect *pRect);
 	void DoIconColor(int ImageId, int SpriteId, const CUIRect *pRect, const vec4& Color);
-	int DoButton_GridHeader(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
-	int DoButton_GridHeaderIcon(CButtonContainer *pBC, int ImageID, int SpriteID, const CUIRect *pRect, int Corners);
+	int DoButton_GridHeader(const void *pID, const char *pText, int Checked, CUI::EAlignment Align, const CUIRect *pRect);
 
-	//static void ui_draw_browse_icon(int what, const CUIRect *r);
-	//static void ui_draw_grid_header(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-
-	/*static void ui_draw_checkbox_common(const void *id, const char *text, const char *boxtext, const CUIRect *r, const void *extra);
-	static void ui_draw_checkbox(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-	static void ui_draw_checkbox_number(const void *id, const char *text, int checked, const CUIRect *r, const void *extra);
-	*/
-	int DoEditBox(void *pID, const CUIRect *pRect, char *pStr, unsigned StrSize, float FontSize, float *pOffset, bool Hidden=false, int Corners=CUI::CORNER_ALL);
+	bool DoEditBox(void *pID, const CUIRect *pRect, char *pStr, unsigned StrSize, float FontSize, float *pOffset, bool Hidden=false, int Corners=CUI::CORNER_ALL);
 	void DoEditBoxOption(void *pID, char *pOption, int OptionLength, const CUIRect *pRect, const char *pStr, float VSplitVal, float *pOffset, bool Hidden=false);
-	void DoScrollbarOption(void *pID, int *pOption, const CUIRect *pRect, const char *pStr, float VSplitVal, int Min, int Max, bool infinite=false);
+	void DoScrollbarOption(void *pID, int *pOption, const CUIRect *pRect, const char *pStr, int Min, int Max, IScrollbarScale *pScale = &LinearScrollbarScale, bool Infinite=false);
+	void DoScrollbarOptionLabeled(void *pID, int *pOption, const CUIRect *pRect, const char *pStr, const char* aLabels[], int Num, IScrollbarScale *pScale = &LinearScrollbarScale);
 	float DoDropdownMenu(void *pID, const CUIRect *pRect, const char *pStr, float HeaderHeight, FDropdownCallback pfnCallback);
 	float DoIndependentDropdownMenu(void *pID, const CUIRect *pRect, const char *pStr, float HeaderHeight, FDropdownCallback pfnCallback, bool* pActive);
 	void DoInfoBox(const CUIRect *pRect, const char *pLable, const char *pValue);
-	//static int ui_do_edit_box(void *id, const CUIRect *rect, char *str, unsigned str_size, float font_size, bool hidden=false);
 
 	float DoScrollbarV(const void *pID, const CUIRect *pRect, float Current);
 	float DoScrollbarH(const void *pID, const CUIRect *pRect, float Current);
+	void DoJoystickBar(const CUIRect *pRect, float Current, float Tolerance, bool Active);
 	void DoButton_KeySelect(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect);
 	int DoKeyReader(CButtonContainer *pPC, const CUIRect *pRect, int Key, int Modifier, int* NewModifier);
-
-	//static int ui_do_key_reader(void *id, const CUIRect *rect, int key);
 	void UiDoGetButtons(int Start, int Stop, CUIRect View, float ButtonHeight, float Spacing);
 
-	struct CListboxItem
+
+	class CUIElementBase
 	{
-		int m_Visible;
-		int m_Selected;
-		CUIRect m_Rect;
-		CUIRect m_HitRect;
+	protected:
+		static CMenus *m_pMenus;	// TODO: Refactor in order to remove this reference to menus
+		static CRenderTools *m_pRenderTools;
+		static CUI *m_pUI;
+		static IInput *m_pInput;
+
+	public:
+		static void Init(CMenus *pMenus) { m_pMenus = pMenus; m_pRenderTools = pMenus->RenderTools(); m_pUI = pMenus->UI(); m_pInput = pMenus->Input(); };
 	};
 
-	struct CListBoxState
+	// Scroll region : found in menus_scrollregion.cpp
+	struct CScrollRegionParams
 	{
-		CUIRect m_ListBoxOriginalView;
+		float m_ScrollbarWidth;
+		float m_ScrollbarMargin;
+		float m_SliderMinHeight;
+		float m_ScrollSpeed;
+		vec4 m_ClipBgColor;
+		vec4 m_ScrollbarBgColor;
+		vec4 m_RailBgColor;
+		vec4 m_SliderColor;
+		vec4 m_SliderColorHover;
+		vec4 m_SliderColorGrabbed;
+		int m_Flags;
+
+		enum {
+			FLAG_CONTENT_STATIC_WIDTH = 0x1
+		};
+
+		CScrollRegionParams()
+		{
+			m_ScrollbarWidth = 20;
+			m_ScrollbarMargin = 5;
+			m_SliderMinHeight = 25;
+			m_ScrollSpeed = 5;
+			m_ClipBgColor = vec4(0.0f, 0.0f, 0.0f, 0.25f);
+			m_ScrollbarBgColor = vec4(0.0f, 0.0f, 0.0f, 0.25f);
+			m_RailBgColor = vec4(1.0f, 1.0f, 1.0f, 0.25f);
+			m_SliderColor = vec4(0.8f, 0.8f, 0.8f, 1.0f);
+			m_SliderColorHover = vec4(1.0f, 1.0f, 1.0f, 1.0f);
+			m_SliderColorGrabbed = vec4(0.9f, 0.9f, 0.9f, 1.0f);
+			m_Flags = 0;
+		}
+	};
+
+	/*
+	Usage:
+		-- Initialization --
+		static CScrollRegion s_ScrollRegion;
+		vec2 ScrollOffset(0, 0);
+		s_ScrollRegion.Begin(&ScrollRegionRect, &ScrollOffset);
+		Content = ScrollRegionRect;
+		Content.y += ScrollOffset.y;
+
+		-- "Register" your content rects --
+		CUIRect Rect;
+		Content.HSplitTop(SomeValue, &Rect, &Content);
+		s_ScrollRegion.AddRect(Rect);
+
+		-- [Optional] Knowing if a rect is clipped --
+		s_ScrollRegion.IsRectClipped(Rect);
+
+		-- [Optional] Scroll to a rect (to the last added rect)--
+		...
+		s_ScrollRegion.AddRect(Rect);
+		s_ScrollRegion.ScrollHere(Option);
+
+		-- End --
+		s_ScrollRegion.End();
+	*/
+	// Instances of CScrollRegion must be static, as member addresses are used as UI item IDs
+	class CScrollRegion : private CUIElementBase
+	{
+	private:
+		float m_ScrollY;
+		float m_ContentH;
+		float m_RequestScrollY; // [0, ContentHeight]
+		CUIRect m_ClipRect;
+		CUIRect m_RailRect;
+		CUIRect m_LastAddedRect; // saved for ScrollHere()
+		vec2 m_MouseGrabStart;
+		vec2 m_ContentScrollOff;
+		CScrollRegionParams m_Params;
+
+	public:
+		enum {
+			SCROLLHERE_KEEP_IN_VIEW=0,
+			SCROLLHERE_TOP,
+			SCROLLHERE_BOTTOM,
+		};
+
+		CScrollRegion();
+		void Begin(CUIRect* pClipRect, vec2* pOutOffset, const CScrollRegionParams* pParams = 0);
+		void End();
+		void AddRect(CUIRect Rect);
+		void ScrollHere(int Option = CScrollRegion::SCROLLHERE_KEEP_IN_VIEW);
+		bool IsRectClipped(const CUIRect& Rect) const;
+		bool IsScrollbarShown() const;
+	};
+
+	// Listbox : found in menus_listbox.cpp
+	struct CListboxItem
+	{
+		bool m_Visible;
+		bool m_Selected;
+		CUIRect m_Rect;
+	};
+
+	// Instances of CListBox must be static, as member addresses are used as UI item IDs
+	class CListBox : private CUIElementBase
+	{
+	private:
 		CUIRect m_ListBoxView;
 		float m_ListBoxRowHeight;
 		int m_ListBoxItemIndex;
 		int m_ListBoxSelectedIndex;
 		int m_ListBoxNewSelected;
+		int m_ListBoxNewSelOffset;
+		int m_ListBoxUpdateScroll;
 		int m_ListBoxDoneEvents;
 		int m_ListBoxNumItems;
 		int m_ListBoxItemsPerRow;
-		float m_ListBoxScrollValue;
 		bool m_ListBoxItemActivated;
+		const char *m_pBottomText;
+		float m_FooterHeight;
+		CScrollRegion m_ScrollRegion;
+		vec2 m_ScrollOffset;
+		char m_aFilterString[64];
+		float m_OffsetFilter;
 
-		CListBoxState()
-		{
-			m_ListBoxScrollValue = 0;
-		}
+	protected:
+		CListboxItem DoNextRow();
+
+	public:
+		CListBox();
+
+		void DoHeader(const CUIRect *pRect, const char *pTitle, float HeaderHeight = 20.0f, float Spacing = 2.0f);
+		bool DoFilter(float FilterHeight = 20.0f, float Spacing = 2.0f);
+		void DoFooter(const char *pBottomText, float FooterHeight = 20.0f); // call before DoStart to create a footer
+		void DoStart(float RowHeight, int NumItems, int ItemsPerRow, int SelectedIndex,
+					const CUIRect *pRect = 0, bool Background = true, bool *pActive = 0);
+		CListboxItem DoNextItem(const void *pID, bool Selected = false, bool *pActive = 0);
+		int DoEnd();
+		bool FilterMatches(const char *pNeedle) const;
+		bool WasItemActivated() const { return m_ListBoxItemActivated; };
 	};
 
-	void UiDoListboxHeader(CListBoxState* pState, const CUIRect *pRect, const char *pTitle, float HeaderHeight, float Spacing);
-	void UiDoListboxStart(CListBoxState* pState, const void *pID, float RowHeight, const char *pBottomText, int NumItems,
-						int ItemsPerRow, int SelectedIndex, const CUIRect *pRect=0, bool Background=true);
-	CListboxItem UiDoListboxNextItem(CListBoxState* pState, const void *pID, bool Selected = false, bool* pActive = NULL);
-	CListboxItem UiDoListboxNextRow(CListBoxState* pState);
-	int UiDoListboxEnd(CListBoxState* pState, bool *pItemActivated);
-
-	//static void demolist_listdir_callback(const char *name, int is_dir, void *user);
-	//static void demolist_list_callback(const CUIRect *rect, int index, void *user);
 
 	enum
 	{
@@ -187,15 +335,18 @@ private:
 	int m_Popup;
 	int m_ActivePage;
 	int m_MenuPage;
+	int m_MenuPageOld;
 	bool m_MenuActive;
-	int m_SidebarTab;
-	bool m_SidebarActive;
-	bool m_ShowServerDetails;
 	bool m_UseMouseButtons;
 	vec2 m_MousePos;
 	vec2 m_PrevMousePos;
+	bool m_CursorActive;
+	bool m_PrevCursorActive;
 	bool m_PopupActive;
 	int m_ActiveListBox;
+	bool m_SkinModified;
+	bool m_KeyReaderWasActive;
+	bool m_KeyReaderIsActive;
 
 	// images
 	struct CMenuImage
@@ -221,7 +372,7 @@ private:
 		bool m_HasDay;
 		bool m_HasNight;
 		IGraphics::CTextureHandle m_IconTexture;
-		bool operator<(const CTheme &Other) { return m_Name < Other.m_Name; }
+		bool operator<(const CTheme &Other) const { return m_Name < Other.m_Name; }
 	};
 	sorted_array<CTheme> m_lThemes;
 
@@ -234,9 +385,8 @@ private:
 	public:
 		enum
 		{
-			GAMEICON_FULL=0,
-			GAMEICON_ON,
-			GAMEICON_OFF,
+			GAMEICON_SIZE=64,
+			GAMEICON_OLDHEIGHT=192,
 		};
 		CGameIcon() {};
 		CGameIcon(const char *pName) : m_Name(pName) {}
@@ -246,15 +396,11 @@ private:
 	};
 	array<CGameIcon> m_lGameIcons;
 	IGraphics::CTextureHandle m_GameIconDefault;
-	void DoGameIcon(const char *pName, const CUIRect *pRect, int Type);
+	void DoGameIcon(const char *pName, const CUIRect *pRect);
 	static int GameIconScan(const char *pName, int IsDir, int DirType, void *pUser);
 
 	int64 m_LastInput;
 
-	// loading
-	int m_LoadCurrent;
-	int m_LoadTotal;
-
 	//
 	char m_aMessageTopic[512];
 	char m_aMessageBody[512];
@@ -267,17 +413,14 @@ private:
 	static float ms_ButtonHeight;
 	static float ms_ListheaderHeight;
 	static float ms_FontmodHeight;
-	static float ms_BackgroundAlpha;
 
 	// for settings
 	bool m_NeedRestartPlayer;
-	bool m_NeedRestartTee;
 	bool m_NeedRestartGraphics;
 	bool m_NeedRestartSound;
 	int m_TeePartSelected;
 	char m_aSaveSkinName[24];
 
-	void SaveSkinfile();
 	bool m_RefreshSkinSelector;
 	const CSkins::CSkin *m_pSelectedSkin;
 
@@ -294,9 +437,13 @@ private:
 	int m_DownloadLastCheckSize;
 	float m_DownloadSpeed;
 
+	// for password popup
+	char m_aPasswordPopupServerAddress[256];
+
 	// for call vote
 	int m_CallvoteSelectedOption;
 	int m_CallvoteSelectedPlayer;
+	char m_aFilterString[VOTE_REASON_LENGTH];
 	char m_aCallvoteReason[VOTE_REASON_LENGTH];
 
 	// for callbacks
@@ -314,14 +461,19 @@ private:
 		bool m_Valid;
 		CDemoHeader m_Info;
 
-		bool operator<(const CDemoItem &Other) { return !str_comp(m_aFilename, "..") ? true : !str_comp(Other.m_aFilename, "..") ? false :
-														m_IsDir && !Other.m_IsDir ? true : !m_IsDir && Other.m_IsDir ? false :
-														str_comp_filenames(m_aFilename, Other.m_aFilename) < 0; }
+		bool operator<(const CDemoItem &Other) const
+		{
+			return !str_comp(m_aFilename, "..") ? true
+				: !str_comp(Other.m_aFilename, "..") ? false
+				: m_IsDir && !Other.m_IsDir ? true
+				: !m_IsDir && Other.m_IsDir ? false
+				: str_comp_filenames(m_aFilename, Other.m_aFilename) < 0;
+		}
 	};
 
 	sorted_array<CDemoItem> m_lDemos;
-	char m_aCurrentDemoFolder[256];
-	char m_aCurrentDemoFile[64];
+	char m_aCurrentDemoFolder[IO_MAX_PATH_LENGTH];
+	char m_aCurrentDemoFile[IO_MAX_PATH_LENGTH];
 	int m_DemolistSelectedIndex;
 	bool m_DemolistSelectedIsDir;
 	int m_DemolistStorageType;
@@ -347,17 +499,14 @@ private:
 			m_pServerInfo = 0;
 		}
 
-		bool operator<(const CFriendItem &Other)
+		bool operator<(const CFriendItem &Other) const
 		{
 			if(m_aName[0] && !Other.m_aName[0])
 				return true;
 			if(!m_aName[0] && Other.m_aName[0])
 				return false;
 			int Result = str_comp_nocase(m_aName, Other.m_aName);
-			if(Result < 0 || (Result == 0 && str_comp_nocase(m_aClan, Other.m_aClan) < 0))
-				return true;
-			
-			return false;
+			return Result < 0 || (Result == 0 && str_comp_nocase(m_aClan, Other.m_aClan) < 0);
 		}
 	};
 
@@ -373,20 +522,21 @@ private:
 
 	void FriendlistOnUpdate();
 
+
+	// server browser
 	class CBrowserFilter
 	{
 		bool m_Extended;
 		int m_Custom;
 		char m_aName[64];
 		int m_Filter;
-		class IServerBrowser *m_pServerBrowser;
+		IServerBrowser *m_pServerBrowser;
 
-		static class CServerFilterInfo ms_FilterStandard;
-		static class CServerFilterInfo ms_FilterFavorites;
-		static class CServerFilterInfo ms_FilterAll;
+		static CServerFilterInfo ms_FilterStandard;
+		static CServerFilterInfo ms_FilterFavorites;
+		static CServerFilterInfo ms_FilterAll;
 
 	public:
-
 		enum
 		{
 			FILTER_CUSTOM=0,
@@ -394,6 +544,7 @@ private:
 			FILTER_STANDARD,
 			FILTER_FAVORITES,
 		};
+
 		// buttons var
 		int m_SwitchButton;
 		int m_aButtonID[3];
@@ -410,23 +561,23 @@ private:
 
 		int NumSortedServers() const;
 		int NumPlayers() const;
-		const CServerInfo *SortedGet(int Index) const;
-		const void *ID(int Index) const;
+		const CServerInfo* SortedGet(int Index) const;
+		const void* ID(int Index) const;
 
 		void Reset();
-		void GetFilter(class CServerFilterInfo *pFilterInfo) const;
-		void SetFilter(const class CServerFilterInfo *pFilterInfo);
+		void GetFilter(CServerFilterInfo *pFilterInfo) const;
+		void SetFilter(const CServerFilterInfo *pFilterInfo);
 	};
 
 	array<CBrowserFilter> m_lFilters;
 
-	int m_SelectedFilter;
 	int m_RemoveFilterIndex;
 
 	void LoadFilters();
 	void SaveFilters();
 	void RemoveFilter(int FilterIndex);
 	void Move(bool Up, int Filter);
+	void InitDefaultFilters();
 
 	class CInfoOverlay
 	{
@@ -448,29 +599,6 @@ private:
 	CInfoOverlay m_InfoOverlay;
 	bool m_InfoOverlayActive;
 
-	class CServerEntry
-	{
-	public:
-		int m_Filter;
-		int m_Index;
-	};
-
-	CServerEntry m_SelectedServer;
-
-	enum
-	{
-		FIXED=1,
-		SPACER=2,
-
-		COL_BROWSER_FLAG=0,
-		COL_BROWSER_NAME,
-		COL_BROWSER_GAMETYPE,
-		COL_BROWSER_MAP,
-		COL_BROWSER_PLAYERS,
-		COL_BROWSER_PING,
-		NUM_BROWSER_COLS,
-	};
-
 	struct CColumn
 	{
 		int m_ID;
@@ -481,44 +609,90 @@ private:
 		int m_Flags;
 		CUIRect m_Rect;
 		CUIRect m_Spacer;
+		CUI::EAlignment m_Align;
 	};
 
+	enum
+	{
+		COL_BROWSER_FLAG=0,
+		COL_BROWSER_NAME,
+		COL_BROWSER_GAMETYPE,
+		COL_BROWSER_MAP,
+		COL_BROWSER_PLAYERS,
+		COL_BROWSER_PING,
+		NUM_BROWSER_COLS,
+
+		ADDR_SELECTION_CHANGE = 1,
+		ADDR_SELECTION_RESET_SERVER_IF_NOT_FOUND = 2,
+		ADDR_SELECTION_REVEAL = 4,
+		ADDR_SELECTION_UPDATE_ADDRESS = 8,
+	};
+	int m_SidebarTab;
+	bool m_SidebarActive;
+	bool m_ShowServerDetails;
+	int m_LastBrowserType; // -1 if not initialized
+	int m_aSelectedFilters[IServerBrowser::NUM_TYPES]; // -1 if none selected, -2 if not initialized
+	int m_aSelectedServers[IServerBrowser::NUM_TYPES]; // -1 if none selected
+	int m_AddressSelection;
 	static CColumn ms_aBrowserCols[NUM_BROWSER_COLS];
 
+	CBrowserFilter* GetSelectedBrowserFilter()
+	{
+		const int Tab = ServerBrowser()->GetType();
+		if(m_aSelectedFilters[Tab] == -1)
+			return 0;
+		return &m_lFilters[m_aSelectedFilters[Tab]];
+	}
+
+	const CServerInfo* GetSelectedServerInfo()
+	{
+		CBrowserFilter* pSelectedFilter = GetSelectedBrowserFilter();
+		if(!pSelectedFilter)
+			return 0;
+		const int Tab = ServerBrowser()->GetType();
+		if(m_aSelectedServers[Tab] < 0 || m_aSelectedServers[Tab] >= pSelectedFilter->NumSortedServers())
+			return 0;
+		return pSelectedFilter->SortedGet(m_aSelectedServers[Tab]);
+	}
+
+	void UpdateServerBrowserAddress();
+	const char *GetServerBrowserAddress();
+	void SetServerBrowserAddress(const char *pAddress);
+	void ServerBrowserFilterOnUpdate();
+	void ServerBrowserSortingOnUpdate();
+
+
+	// video settings
 	enum
 	{
 		MAX_RESOLUTIONS=256,
 	};
-
 	CVideoMode m_aModes[MAX_RESOLUTIONS];
 	int m_NumModes;
-
 	struct CVideoFormat
 	{
 		int m_WidthValue;
 		int m_HeightValue;
 	};
-
-	CVideoFormat m_aVideoFormats[MAX_RESOLUTIONS];
 	sorted_array<CVideoMode> m_lRecommendedVideoModes;
 	sorted_array<CVideoMode> m_lOtherVideoModes;
-	int m_NumVideoFormats;
-	int m_CurrentVideoFormat;
-	void UpdateVideoFormats();
 	void UpdatedFilteredVideoModes();
 	void UpdateVideoModeSettings();
 
 	// found in menus.cpp
 	int Render();
-	//void render_background();
-	//void render_loading(float percent);
 	void RenderMenubar(CUIRect r);
 	void RenderNews(CUIRect MainView);
 	void RenderBackButton(CUIRect MainView);
+	inline float GetListHeaderHeight() const { return ms_ListheaderHeight + (Config()->m_UiWideview ? 3.0f : 0.0f); }
+	inline float GetListHeaderHeightFactor() const { return 1.0f + (Config()->m_UiWideview ? (3.0f/ms_ListheaderHeight) : 0.0f); }
+	static void ConchainUpdateMusicState(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	void UpdateMusicState();
 
 	// found in menus_demo.cpp
 	void RenderDemoPlayer(CUIRect MainView);
 	void RenderDemoList(CUIRect MainView);
+	float RenderDemoDetails(CUIRect View);
 
 	// found in menus_start.cpp
 	void RenderStartMenu(CUIRect MainView);
@@ -531,10 +705,9 @@ private:
 	void HandleCallvote(int Page, bool Force);
 	void RenderServerControl(CUIRect MainView);
 	void RenderServerControlKick(CUIRect MainView, bool FilterSpectators);
-	void RenderServerControlServer(CUIRect MainView);
+	bool RenderServerControlServer(CUIRect MainView);
 
 	// found in menus_browser.cpp
-	// int m_ScrollOffset;
 	void RenderServerbrowserServerList(CUIRect View);
 	void RenderServerbrowserSidebar(CUIRect View);
 	void RenderServerbrowserFriendTab(CUIRect View);
@@ -542,18 +715,18 @@ private:
 	void RenderServerbrowserInfoTab(CUIRect View);
 	void RenderServerbrowserFriendList(CUIRect View);
 	void RenderDetailInfo(CUIRect View, const CServerInfo *pInfo);
-	void RenderDetailScoreboard(CUIRect View, const CServerInfo *pInfo, int RowCount, vec4 TextColor = vec4(1,1,1,1));
+	void RenderDetailScoreboard(CUIRect View, const CServerInfo *pInfo, int RowCount, vec4 TextColor, vec4 TextOutlineColor);
 	void RenderServerbrowserServerDetail(CUIRect View, const CServerInfo *pInfo);
-	//void RenderServerbrowserFriends(CUIRect View);
 	void RenderServerbrowserBottomBox(CUIRect View);
 	void RenderServerbrowserOverlay();
 	void RenderFilterHeader(CUIRect View, int FilterIndex);
 	int DoBrowserEntry(const void *pID, CUIRect View, const CServerInfo *pEntry, const CBrowserFilter *pFilter, bool Selected);
 	void RenderServerbrowser(CUIRect MainView);
+	static void ConchainConnect(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainFriendlistUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainServerbrowserUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
-	static void ConchainToggleMusic(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
-	void DoFriendListEntry(CUIRect *pView, CFriendItem *pFriend, const void *pID, const CFriendInfo *pFriendInfo, const CServerInfo *pServerInfo, bool Checked, bool Clan = false);
+	static void ConchainServerbrowserSortingUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
+	void DoFriendListEntry(CUIRect *pView, CFriendItem *pFriend, const void *pID, const CContactInfo *pFriendInfo, const CServerInfo *pServerInfo, bool Checked, bool Clan = false);
 	void SetOverlay(int Type, float x, float y, const void *pData);
 	void UpdateFriendCounter(const CServerInfo *pEntry);
 	void UpdateFriends();
@@ -564,6 +737,7 @@ private:
 	void RenderHSLPicker(CUIRect Picker);
 	void RenderSkinSelection(CUIRect MainView);
 	void RenderSkinPartSelection(CUIRect MainView);
+	void RenderSkinPartPalette(CUIRect MainView);
 	void RenderSettingsGeneral(CUIRect MainView);
 	void RenderSettingsPlayer(CUIRect MainView);
 	void RenderSettingsTee(CUIRect MainView);
@@ -574,27 +748,42 @@ private:
 	void RenderSettingsSound(CUIRect MainView);
 	void RenderSettings(CUIRect MainView);
 
-	bool DoResolutionList(CUIRect* pRect, CListBoxState* pListBoxState,
+	bool DoResolutionList(CUIRect* pRect, CListBox* pListBox,
 						  const sorted_array<CVideoMode>& lModes);
 
-	// found in menu_callback.cpp
-	static float RenderSettingsControlsMovement(CUIRect View, void *pUser);
-	static float RenderSettingsControlsWeapon(CUIRect View, void *pUser);
-	static float RenderSettingsControlsVoting(CUIRect View, void *pUser);
-	static float RenderSettingsControlsChat(CUIRect View, void *pUser);
-	static float RenderSettingsControlsMisc(CUIRect View, void *pUser);
+	// found in menus_callback.cpp
+	float RenderSettingsControlsMouse(CUIRect View);
+	float RenderSettingsControlsJoystick(CUIRect View);
+	float RenderSettingsControlsMovement(CUIRect View);
+	float RenderSettingsControlsWeapon(CUIRect View);
+	float RenderSettingsControlsVoting(CUIRect View);
+	float RenderSettingsControlsChat(CUIRect View);
+	float RenderSettingsControlsScoreboard(CUIRect View);
+	float RenderSettingsControlsStats(CUIRect View);
+	float RenderSettingsControlsMisc(CUIRect View);
+
+	void DoJoystickAxisPicker(CUIRect View);
 
 	void SetActive(bool Active);
 
 	void InvokePopupMenu(void *pID, int Flags, float X, float Y, float W, float H, int (*pfnFunc)(CMenus *pMenu, CUIRect Rect), void *pExtra=0);
 	void DoPopupMenu();
 
-	IGraphics::CTextureHandle m_TextureBlob;
-
-	void ToggleMusic();
+	// loading
+	int m_LoadCurrent;
+	int m_LoadTotal;
 
 	void SetMenuPage(int NewPage);
+
+	bool CheckHotKey(int Key) const;
+
+	void RenderBackground(float Time);
+	void RenderBackgroundShadow(const CUIRect *pRect, bool TopToBottom);
 public:
+	void InitLoading(int TotalWorkAmount);
+	void RenderLoading(int WorkedAmount = 0);
+	bool IsBackgroundNeeded() const;
+
 	struct CSwitchTeamInfo
 	{
 		char m_aNotification[128];
@@ -602,7 +791,6 @@ public:
 		int m_TimeLeft;
 	};
 	void GetSwitchTeamInfo(CSwitchTeamInfo *pInfo);
-	void RenderBackground();
 
 	void UseMouseButtons(bool Use) { m_UseMouseButtons = Use; }
 
@@ -610,10 +798,9 @@ public:
 
 	CMenus();
 
-	void RenderLoading();
-
 	bool IsActive() const { return m_MenuActive; }
 
+	virtual int GetInitAmount() const;
 	virtual void OnInit();
 
 	virtual void OnConsoleInit();
