@@ -15,29 +15,36 @@
 #include <game/client/ui.h>
 #include <game/client/components/countryflags.h>
 #include <game/client/components/motd.h>
+#include <game/client/components/stats.h>
 
 #include "menus.h"
+#include "stats.h"
 #include "scoreboard.h"
+#include "stats.h"
 
 
 CScoreboard::CScoreboard()
 {
-	m_PlayerLines = 0;
 	OnReset();
 }
 
 void CScoreboard::ConKeyScoreboard(IConsole::IResult *pResult, void *pUserData)
 {
-	((CScoreboard *)pUserData)->m_Active = pResult->GetInteger(0) != 0;
+	CScoreboard *pScoreboard = (CScoreboard *)pUserData;
+	int Result = pResult->GetInteger(0);
+	if(!Result)
+	{
+		pScoreboard->m_Activate = false;
+		pScoreboard->m_Active = false;
+	}
+	else if(!pScoreboard->m_Active)
+		pScoreboard->m_Activate = true;	
 }
 
 void CScoreboard::OnReset()
 {
 	m_Active = false;
-	m_PlayerLines = 0;
-	m_SkipPlayerStatsReset = false;
-	for(int i = 0; i < MAX_CLIENTS; i++)
-		ResetPlayerStats(i);
+	m_Activate = false;
 }
 
 void CScoreboard::OnRelease()
@@ -64,21 +71,21 @@ void CScoreboard::RenderGoals(float x, float y, float w)
 	{
 		char aBuf[64];
 		str_format(aBuf, sizeof(aBuf), "%s: %d", Localize("Score limit"), m_pClient->m_GameInfo.m_ScoreLimit);
-		TextRender()->Text(0, x+10.0f, y, 12.0f, aBuf, -1);
+		TextRender()->Text(0, x+10.0f, y, 12.0f, aBuf, -1.0f);
 	}
 	if(m_pClient->m_GameInfo.m_TimeLimit)
 	{
 		char aBuf[64];
 		str_format(aBuf, sizeof(aBuf), Localize("Time limit: %d min"), m_pClient->m_GameInfo.m_TimeLimit);
-		float tw = TextRender()->TextWidth(0, 12.0f, aBuf, -1);
-		TextRender()->Text(0, x+w/2-tw/2, y, 12.0f, aBuf, -1);
+		float tw = TextRender()->TextWidth(0, 12.0f, aBuf, -1, -1.0f);
+		TextRender()->Text(0, x+w/2-tw/2, y, 12.0f, aBuf, -1.0f);
 	}
 	if(m_pClient->m_GameInfo.m_MatchNum && m_pClient->m_GameInfo.m_MatchCurrent)
 	{
 		char aBuf[64];
 		str_format(aBuf, sizeof(aBuf), "%s %d/%d", Localize("Match", "rounds (scoreboard)"), m_pClient->m_GameInfo.m_MatchCurrent, m_pClient->m_GameInfo.m_MatchNum);
-		float tw = TextRender()->TextWidth(0, 12.0f, aBuf, -1);
-		TextRender()->Text(0, x+w-tw-10.0f, y, 12.0f, aBuf, -1);
+		float tw = TextRender()->TextWidth(0, 12.0f, aBuf, -1, -1.0f);
+		TextRender()->Text(0, x+w-tw-10.0f, y, 12.0f, aBuf, -1.0f);
 	}
 }
 
@@ -94,7 +101,7 @@ float CScoreboard::RenderSpectators(float x, float y, float w)
 	char aBuf[64];
 	char SpectatorBuf[64];
 	str_format(SpectatorBuf, sizeof(SpectatorBuf), "%s (%d):", Localize("Spectators"), NumSpectators);
-	float tw = TextRender()->TextWidth(0, 12.0f, SpectatorBuf, -1);
+	float tw = TextRender()->TextWidth(0, 12.0f, SpectatorBuf, -1, -1.0f);
 
 	// do all the text without rendering it once
 	CTextCursor Cursor;
@@ -111,7 +118,7 @@ float CScoreboard::RenderSpectators(float x, float y, float w)
 
 		if(Multiple)
 			TextRender()->TextEx(&Cursor, ", ", -1);
-		if(g_Config.m_ClShowUserId && Cursor.m_LineCount <= Cursor.m_MaxLines)
+		if(Config()->m_ClShowUserId && Cursor.m_LineCount <= Cursor.m_MaxLines)
 		{
 			Cursor.m_X += Cursor.m_FontSize;
 		}
@@ -149,7 +156,7 @@ float CScoreboard::RenderSpectators(float x, float y, float w)
 
 		if(Multiple)
 			TextRender()->TextEx(&Cursor, ", ", -1);
-		if(g_Config.m_ClShowUserId && Cursor.m_LineCount <= Cursor.m_MaxLines)
+		if(Config()->m_ClShowUserId && Cursor.m_LineCount <= Cursor.m_MaxLines)
 		{
 			RenderTools()->DrawClientID(TextRender(), &Cursor, i);
 		}
@@ -177,6 +184,8 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 	const CGameClient::CSnapState& Snap = m_pClient->m_Snap;
 	const bool ReadyMode = Snap.m_pGameData && (Snap.m_pGameData->m_GameStateFlags&(GAMESTATEFLAG_STARTCOUNTDOWN|GAMESTATEFLAG_PAUSED|GAMESTATEFLAG_WARMUP)) && Snap.m_pGameData->m_GameStateEndTick == 0;
 
+	bool Race = m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_RACE;
+
 	float HeadlineHeight = 40.0f;
 	float TitleFontsize = 20.0f;
 	float HeadlineFontsize = 12.0f;
@@ -185,14 +194,14 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 	float Spacing = 2.0f;
 	float PingOffset = x+Spacing, PingLength = 35.0f;
 	float CountryFlagOffset = PingOffset+PingLength, CountryFlagLength = 20.f;
-	float IdSize = g_Config.m_ClShowUserId ? LineHeight : 0.0f;
+	float IdSize = Config()->m_ClShowUserId ? LineHeight : 0.0f;
 	float ReadyLength = ReadyMode ? 10.f : 0.f;
 	float TeeOffset = CountryFlagOffset+CountryFlagLength+4.0f, TeeLength = 25*TeeSizeMod;
 	float NameOffset = CountryFlagOffset+CountryFlagLength+IdSize, NameLength = 128.0f-IdSize/2-ReadyLength;
 	float ClanOffset = NameOffset+NameLength+ReadyLength, ClanLength = 88.0f-IdSize/2;
-	float KillOffset = ClanOffset+ClanLength, KillLength = 24.0f;
-	float DeathOffset = KillOffset+KillLength, DeathLength = 24.0f;
-	float ScoreOffset = DeathOffset+DeathLength, ScoreLength = 35.0f;
+	float KillOffset = ClanOffset+ClanLength, KillLength = Race ? 0.0f : 24.0f;
+	float DeathOffset = KillOffset+KillLength, DeathLength = Race ? 0.0f : 24.0f;
+	float ScoreOffset = DeathOffset+DeathLength, ScoreLength = Race ? 83.0f : 35.0f;
 	float tw = 0.0f;
 
 	bool NoTitle = pTitle? false : true;
@@ -200,11 +209,13 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 	// count players
 	dbg_assert(Team == TEAM_RED || Team == TEAM_BLUE, "Unknown team id");
 	int NumPlayers = m_pClient->m_GameInfo.m_aTeamSize[Team];
-	m_PlayerLines = max(m_PlayerLines, NumPlayers);
+	int PlayerLines = NumPlayers;
+	if(m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_TEAMS)
+		PlayerLines = max(m_pClient->m_GameInfo.m_aTeamSize[Team^1], PlayerLines);
 
 	// clamp to 16
-	if(m_PlayerLines > 16)
-		m_PlayerLines = 16;
+	if(PlayerLines > 16)
+		PlayerLines = 16;
 
 	char aBuf[128] = {0};
 
@@ -226,10 +237,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 		if(m_pClient->m_Snap.m_pGameData->m_GameStateFlags&GAMESTATEFLAG_GAMEOVER)
 			pTitle = Localize("Game over");
 		else if(m_pClient->m_Snap.m_pGameData->m_GameStateFlags&GAMESTATEFLAG_ROUNDOVER)
-		{
 			pTitle = Localize("Round over");
-			m_SkipPlayerStatsReset = true;
-		}
 		else
 			pTitle = Localize("Scoreboard");
 	}
@@ -243,92 +251,122 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 
 	if(Align == -1)
 	{
-		tw = TextRender()->TextWidth(0, TitleFontsize, pTitle, -1);
-		TextRender()->Text(0, x+20.0f, y+5.0f, TitleFontsize, pTitle, -1);
+		tw = TextRender()->TextWidth(0, TitleFontsize, pTitle, -1, -1.0f);
+		TextRender()->Text(0, x+20.0f, y+5.0f, TitleFontsize, pTitle, -1.0f);
 		if(!NoTitle)
 		{
 			str_format(aBuf, sizeof(aBuf), " (%d)", NumPlayers);
 			TextRender()->TextColor(1.0f, 1.0f, 1.0f, 0.5f);
-			TextRender()->Text(0, x+20.0f+tw, y+5.0f, TitleFontsize, aBuf, -1);
+			TextRender()->Text(0, x+20.0f+tw, y+5.0f, TitleFontsize, aBuf, -1.0f);
 			TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
 		}
 	}
 	else
 	{
-		tw = TextRender()->TextWidth(0, TitleFontsize, pTitle, -1);
+		tw = TextRender()->TextWidth(0, TitleFontsize, pTitle, -1, -1.0f);
 		if(!NoTitle)
 		{
 			str_format(aBuf, sizeof(aBuf), "(%d) ", NumPlayers);
-			float PlayersTextWidth = TextRender()->TextWidth(0, TitleFontsize, aBuf, -1);
+			float PlayersTextWidth = TextRender()->TextWidth(0, TitleFontsize, aBuf, -1, -1.0f);
 			TextRender()->TextColor(1.0f, 1.0f, 1.0f, 0.5f);
-			TextRender()->Text(0, x+w-tw-PlayersTextWidth-20.0f, y+5.0f, TitleFontsize, aBuf, -1);
+			TextRender()->Text(0, x+w-tw-PlayersTextWidth-20.0f, y+5.0f, TitleFontsize, aBuf, -1.0f);
 			TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
 		}
-		tw = TextRender()->TextWidth(0, TitleFontsize, pTitle, -1);
-		TextRender()->Text(0, x+w-tw-20.0f, y+5.0f, TitleFontsize, pTitle, -1);
+		tw = TextRender()->TextWidth(0, TitleFontsize, pTitle, -1, -1.0f);
+		TextRender()->Text(0, x+w-tw-20.0f, y+5.0f, TitleFontsize, pTitle, -1.0f);
 	}
 
-	if(m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_TEAMS)
+	if(Race)
 	{
-		int Score = Team == TEAM_RED ? m_pClient->m_Snap.m_pGameDataTeam->m_TeamscoreRed : m_pClient->m_Snap.m_pGameDataTeam->m_TeamscoreBlue;
-		str_format(aBuf, sizeof(aBuf), "%d", Score);
+		if(m_pClient->m_Snap.m_pGameDataRace && Team != TEAM_BLUE)
+		{
+			float MapRecordFontsize = 16.0f;
+			const char *pMapRecordStr = Localize("Map record");
+			FormatTime(aBuf, sizeof(aBuf), m_pClient->m_Snap.m_pGameDataRace->m_BestTime, m_pClient->RacePrecision());
+			if(Align == -1)
+			{
+				tw = TextRender()->TextWidth(0, HeadlineFontsize, pMapRecordStr, -1, -1.0f);
+				TextRender()->Text(0, x+w-tw-20.0f, y+3.0f, HeadlineFontsize, pMapRecordStr, -1.0f);
+				tw = TextRender()->TextWidth(0, MapRecordFontsize, aBuf, -1, -1.0f);
+				TextRender()->Text(0, x+w-tw-20.0f, y+HeadlineFontsize+3.0f, MapRecordFontsize, aBuf, -1.0f);
+			}
+			else
+			{
+				TextRender()->Text(0, x+20.0f, y+3.0f, HeadlineFontsize, pMapRecordStr, -1.0f);
+				TextRender()->Text(0, x+20.0f, y+HeadlineFontsize+3.0f, MapRecordFontsize, aBuf, -1.0f);
+			}
+		}
 	}
 	else
 	{
-		if(m_pClient->m_Snap.m_SpecInfo.m_Active && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID >= 0 &&
-			m_pClient->m_Snap.m_paPlayerInfos[m_pClient->m_Snap.m_SpecInfo.m_SpectatorID])
+		if(m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_TEAMS)
 		{
-			int Score = m_pClient->m_Snap.m_paPlayerInfos[m_pClient->m_Snap.m_SpecInfo.m_SpectatorID]->m_Score;
+			int Score = Team == TEAM_RED ? m_pClient->m_Snap.m_pGameDataTeam->m_TeamscoreRed : m_pClient->m_Snap.m_pGameDataTeam->m_TeamscoreBlue;
 			str_format(aBuf, sizeof(aBuf), "%d", Score);
 		}
-		else if(m_pClient->m_Snap.m_pLocalInfo)
+		else
 		{
-			int Score = m_pClient->m_Snap.m_pLocalInfo->m_Score;
-			str_format(aBuf, sizeof(aBuf), "%d", Score);
+			if(m_pClient->m_Snap.m_SpecInfo.m_Active && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID >= 0 &&
+				m_pClient->m_Snap.m_paPlayerInfos[m_pClient->m_Snap.m_SpecInfo.m_SpectatorID])
+			{
+				int Score = m_pClient->m_Snap.m_paPlayerInfos[m_pClient->m_Snap.m_SpecInfo.m_SpectatorID]->m_Score;
+				str_format(aBuf, sizeof(aBuf), "%d", Score);
+			}
+			else if(m_pClient->m_Snap.m_pLocalInfo)
+			{
+				int Score = m_pClient->m_Snap.m_pLocalInfo->m_Score;
+				str_format(aBuf, sizeof(aBuf), "%d", Score);
+			}
 		}
+		if(Align == -1)
+		{
+			tw = TextRender()->TextWidth(0, TitleFontsize, aBuf, -1, -1.0f);
+			TextRender()->Text(0, x+w-tw-20.0f, y+5.0f, TitleFontsize, aBuf, -1.0f);
+		}
+		else
+			TextRender()->Text(0, x+20.0f, y+5.0f, TitleFontsize, aBuf, -1.0f);
 	}
-	if(Align == -1)
-	{
-		tw = TextRender()->TextWidth(0, TitleFontsize, aBuf, -1);
-		TextRender()->Text(0, x+w-tw-20.0f, y+5.0f, TitleFontsize, aBuf, -1);
-	}
-	else
-		TextRender()->Text(0, x+20.0f, y+5.0f, TitleFontsize, aBuf, -1);
 
 	// render headlines
 	y += HeadlineHeight;
 
 	Graphics()->BlendNormal();
 	{
-		CUIRect Rect = {x, y, w, LineHeight*(m_PlayerLines+1)};
+		CUIRect Rect = {x, y, w, LineHeight*(PlayerLines+1)};
 		RenderTools()->DrawRoundRect(&Rect, vec4(0.0f, 0.0f, 0.0f, 0.25f), 5.0f);
 	}
-	if(m_PlayerLines)
+	if(PlayerLines)
 	{
-		CUIRect Rect = {x, y+LineHeight, w, LineHeight*(m_PlayerLines)};
+		CUIRect Rect = {x, y+LineHeight, w, LineHeight*(PlayerLines)};
 		RenderTools()->DrawRoundRect(&Rect, vec4(0.0f, 0.0f, 0.0f, 0.25f), 5.0f);
 	}
 
+	const char *pPingStr = Localize("Ping");
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 0.5f);
-	tw = TextRender()->TextWidth(0, HeadlineFontsize, Localize("Ping"), -1);
-	TextRender()->Text(0, PingOffset+PingLength-tw, y+Spacing, HeadlineFontsize, Localize("Ping"), -1);
+	tw = TextRender()->TextWidth(0, HeadlineFontsize, pPingStr, -1, -1.0f);
+	TextRender()->Text(0, PingOffset+PingLength-tw, y+Spacing, HeadlineFontsize, pPingStr, -1.0f);
 
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
-	TextRender()->Text(0, NameOffset+ TeeLength, y+Spacing, HeadlineFontsize, Localize("Name"), -1);
+	TextRender()->Text(0, NameOffset+ TeeLength, y+Spacing, HeadlineFontsize, Localize("Name"), -1.0f);
 
-	tw = TextRender()->TextWidth(0, HeadlineFontsize, Localize("Clan"), -1);
-	TextRender()->Text(0, ClanOffset+ClanLength/2-tw/2, y+Spacing, HeadlineFontsize, Localize("Clan"), -1);
+	const char *pClanStr = Localize("Clan");
+	tw = TextRender()->TextWidth(0, HeadlineFontsize, pClanStr, -1, -1.0f);
+	TextRender()->Text(0, ClanOffset+ClanLength/2-tw/2, y+Spacing, HeadlineFontsize, pClanStr, -1.0f);
 
-	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 0.5f);
-	tw = TextRender()->TextWidth(0, HeadlineFontsize, "K", -1);
-	TextRender()->Text(0, KillOffset+KillLength/2-tw/2, y+Spacing, HeadlineFontsize, Localize("K"), -1);
+	if(!Race)
+	{
+		TextRender()->TextColor(1.0f, 1.0f, 1.0f, 0.5f);
+		tw = TextRender()->TextWidth(0, HeadlineFontsize, "K", -1, -1.0f);
+		TextRender()->Text(0, KillOffset+KillLength/2-tw/2, y+Spacing, HeadlineFontsize, "K", -1.0f);
 
-	tw = TextRender()->TextWidth(0, HeadlineFontsize, "D", -1);
-	TextRender()->Text(0, DeathOffset+DeathLength/2-tw/2, y+Spacing, HeadlineFontsize, Localize("D"), -1);
+		tw = TextRender()->TextWidth(0, HeadlineFontsize, "D", -1, -1.0f);
+		TextRender()->Text(0, DeathOffset+DeathLength/2-tw/2, y+Spacing, HeadlineFontsize, "D", -1.0f);
+	}
 
+	const char *pScoreStr = Race ? Localize("Time") : Localize("Score");
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
-	tw = TextRender()->TextWidth(0, HeadlineFontsize, Localize("Score"), -1);
-	TextRender()->Text(0, ScoreOffset+ScoreLength/2-tw/2, y+Spacing, HeadlineFontsize, Localize("Score"), -1);
+	tw = TextRender()->TextWidth(0, HeadlineFontsize, pScoreStr, -1, -1.0f);
+	TextRender()->Text(0, ScoreOffset + (Race ? ScoreLength-tw-3.f : ScoreLength/2-tw/2), y+Spacing, HeadlineFontsize, pScoreStr, -1.0f);
 
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
 
@@ -337,16 +375,19 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 	float FontSize = HeadlineFontsize;
 	CTextCursor Cursor;
 
-	int RenderScoreIDs[16] = { -1 };
+	const int MAX_IDS = 16;
+	int RenderScoreIDs[MAX_IDS];
 	int NumRenderScoreIDs = 0;
 	int HoleSizes[2];
+	for(int i = 0; i < MAX_IDS; ++i)
+		RenderScoreIDs[i] = -1;
 
 	// Non vanilla scoreboard, for now, some parts of the scoreboard are omitted
-	if(NumPlayers > 16)
+	if(NumPlayers > MAX_IDS)
 	{
-		for(int RenderDead = 0; RenderDead < 2 && NumRenderScoreIDs < 15; ++RenderDead)
+		for(int RenderDead = 0; RenderDead < 2 && NumRenderScoreIDs < MAX_IDS-1; ++RenderDead)
 		{
-			for(int i = 0; i < MAX_CLIENTS && NumRenderScoreIDs < 15; i++)
+			for(int i = 0; i < MAX_CLIENTS && NumRenderScoreIDs < MAX_IDS-1; i++)
 			{
 				// make sure that we render the correct team
 				const CGameClient::CPlayerInfoItem *pInfo = &m_pClient->m_Snap.m_aInfoByScore[i];
@@ -358,9 +399,9 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 				NumRenderScoreIDs++;
 			}
 		}
-		NumRenderScoreIDs = 16;
-		RenderScoreIDs[15] = -1;
-		HoleSizes[0] = m_pClient->m_GameInfo.m_aTeamSize[Team] - 15;
+		NumRenderScoreIDs = MAX_IDS;
+		RenderScoreIDs[MAX_IDS-1] = -1;
+		HoleSizes[0] = m_pClient->m_GameInfo.m_aTeamSize[Team] - (MAX_IDS-1);
 
 		if(m_pClient->m_LocalClientID != -1 && (m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team == Team || m_pClient->m_Snap.m_SpecInfo.m_Active))
 		{
@@ -384,40 +425,40 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 				}
 			}
 
-			if(Classment < 15) {}
+			if(Classment < MAX_IDS-1) {}
 			else if(Classment == m_pClient->m_GameInfo.m_aTeamSize[Team] - 1)
 			{
-				HoleSizes[0] = Classment - 14;
-				RenderScoreIDs[13] = -1;
-				RenderScoreIDs[14] = TeamScoreIDs[Classment-1];
-				RenderScoreIDs[15] = TeamScoreIDs[Classment];
+				HoleSizes[0] = Classment - MAX_IDS-2;
+				RenderScoreIDs[MAX_IDS-3] = -1;
+				RenderScoreIDs[MAX_IDS-2] = TeamScoreIDs[Classment-1];
+				RenderScoreIDs[MAX_IDS-1] = TeamScoreIDs[Classment];
 			}
 			else if(Classment == m_pClient->m_GameInfo.m_aTeamSize[Team] - 2)
 			{
-				HoleSizes[0] = Classment - 13;
-				RenderScoreIDs[12] = -1;
-				RenderScoreIDs[13] = TeamScoreIDs[Classment-1];
-				RenderScoreIDs[14] = TeamScoreIDs[Classment];
-				RenderScoreIDs[15] = TeamScoreIDs[Classment+1];
+				HoleSizes[0] = Classment - MAX_IDS-3;
+				RenderScoreIDs[MAX_IDS-4] = -1;
+				RenderScoreIDs[MAX_IDS-3] = TeamScoreIDs[Classment-1];
+				RenderScoreIDs[MAX_IDS-2] = TeamScoreIDs[Classment];
+				RenderScoreIDs[MAX_IDS-1] = TeamScoreIDs[Classment+1];
 			}
 			else if(Classment == m_pClient->m_GameInfo.m_aTeamSize[Team] - 3)
 			{
-				HoleSizes[0] = Classment - 12;
-				RenderScoreIDs[11] = -1;
-				RenderScoreIDs[12] = TeamScoreIDs[Classment-1];
-				RenderScoreIDs[13] = TeamScoreIDs[Classment];
-				RenderScoreIDs[14] = TeamScoreIDs[Classment+1];
-				RenderScoreIDs[15] = TeamScoreIDs[Classment+2];
+				HoleSizes[0] = Classment - MAX_IDS-4;
+				RenderScoreIDs[MAX_IDS-5] = -1;
+				RenderScoreIDs[MAX_IDS-4] = TeamScoreIDs[Classment-1];
+				RenderScoreIDs[MAX_IDS-3] = TeamScoreIDs[Classment];
+				RenderScoreIDs[MAX_IDS-2] = TeamScoreIDs[Classment+1];
+				RenderScoreIDs[MAX_IDS-1] = TeamScoreIDs[Classment+2];
 			}
 			else if(Classment < m_pClient->m_GameInfo.m_aTeamSize[Team] - 3)
 			{
-				HoleSizes[0] = Classment - 12;
-				RenderScoreIDs[11] = -1;
-				RenderScoreIDs[12] = TeamScoreIDs[Classment-1];
-				RenderScoreIDs[13] = TeamScoreIDs[Classment];
-				RenderScoreIDs[14] = TeamScoreIDs[Classment+1];
+				HoleSizes[0] = Classment - MAX_IDS-4;
+				RenderScoreIDs[MAX_IDS-5] = -1;
+				RenderScoreIDs[MAX_IDS-4] = TeamScoreIDs[Classment-1];
+				RenderScoreIDs[MAX_IDS-3] = TeamScoreIDs[Classment];
+				RenderScoreIDs[MAX_IDS-2] = TeamScoreIDs[Classment+1];
 				HoleSizes[1] = m_pClient->m_GameInfo.m_aTeamSize[Team] - Classment - 2;
-				RenderScoreIDs[15] = -2;
+				RenderScoreIDs[MAX_IDS-1] = -2;
 			}
 		}
 	}
@@ -425,7 +466,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 	{
 		for(int RenderDead = 0; RenderDead < 2; ++RenderDead)
 		{
-			for(int i = 0; i < MAX_CLIENTS; i++)
+			for(int i = 0; i < MAX_CLIENTS && NumRenderScoreIDs < MAX_IDS; i++)
 			{
 				// make sure that we render the correct team
 				const CGameClient::CPlayerInfoItem *pInfo = &m_pClient->m_Snap.m_aInfoByScore[i];
@@ -473,7 +514,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 			// ping
 			TextRender()->TextColor(TextColor.r, TextColor.g, TextColor.b, 0.5f*ColorAlpha);
 			str_format(aBuf, sizeof(aBuf), "%d", clamp(pInfo->m_pPlayerInfo->m_Latency, 0, 999));
-			tw = TextRender()->TextWidth(0, FontSize, aBuf, -1);
+			tw = TextRender()->TextWidth(0, FontSize, aBuf, -1, -1.0f);
 			TextRender()->SetCursor(&Cursor, PingOffset+PingLength-tw, y+Spacing, FontSize, TEXTFLAG_RENDER);
 			Cursor.m_LineWidth = PingLength;
 			TextRender()->TextEx(&Cursor, aBuf, -1);
@@ -493,7 +534,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 				Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
 				Graphics()->QuadsBegin();
 
-				RenderTools()->SelectSprite(m_pClient->m_aClients[pInfo->m_ClientID].m_Team==TEAM_RED ? SPRITE_FLAG_BLUE : SPRITE_FLAG_RED, SPRITE_FLAG_FLIP_X);
+				RenderTools()->SelectSprite(pInfo->m_ClientID == m_pClient->m_Snap.m_pGameDataFlag->m_FlagCarrierBlue ? SPRITE_FLAG_BLUE : SPRITE_FLAG_RED, SPRITE_FLAG_FLIP_X);
 
 				float Size = LineHeight;
 				IGraphics::CQuadItem QuadItem(TeeOffset+4.0f, y-2.0f-Spacing/2.0f, Size/2.0f, Size);
@@ -512,7 +553,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 					vec4 Color = m_pClient->m_pSkins->GetColorV4(m_pClient->m_pSkins->GetTeamColor(true, 0, m_pClient->m_aClients[pInfo->m_ClientID].m_Team, SKINPART_BODY), false);
 					Graphics()->SetColor(Color.r, Color.g, Color.b, Color.a);
 				}
-				IGraphics::CQuadItem QuadItem(TeeOffset+TeeLength/2 - 10*TeeSizeMod, y+Spacing, 20*TeeSizeMod, 20*TeeSizeMod);
+				IGraphics::CQuadItem QuadItem(TeeOffset+TeeLength/2 - 10*TeeSizeMod, y-2.0f+Spacing, 20*TeeSizeMod, 20*TeeSizeMod);
 				Graphics()->QuadsDrawTL(&QuadItem, 1);
 				Graphics()->QuadsEnd();
 			}
@@ -528,7 +569,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 				TextRender()->TextColor(1.0f, 1.0f, 0.0f, ColorAlpha);
 
 			// id
-			if(g_Config.m_ClShowUserId)
+			if(Config()->m_ClShowUserId)
 			{
 				TextRender()->SetCursor(&Cursor, NameOffset+TeeLength-IdSize+Spacing, y+Spacing, FontSize, TEXTFLAG_RENDER);
 				RenderTools()->DrawClientID(TextRender(), &Cursor, pInfo->m_ClientID);
@@ -551,31 +592,44 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 			TextRender()->TextOutlineColor(OutlineColor.r, OutlineColor.g, OutlineColor.b, OutlineColor.a);
 
 			// clan
-			tw = TextRender()->TextWidth(0, FontSize, m_pClient->m_aClients[pInfo->m_ClientID].m_aClan, -1/*, TEXTFLAG_STOP_AT_END, ClanLength*/);
+			tw = TextRender()->TextWidth(0, FontSize, m_pClient->m_aClients[pInfo->m_ClientID].m_aClan, -1, -1.0f);
 			TextRender()->SetCursor(&Cursor, ClanOffset+ClanLength/2-tw/2, y+Spacing, FontSize, TEXTFLAG_RENDER|TEXTFLAG_STOP_AT_END);
 			Cursor.m_LineWidth = ClanLength;
 			TextRender()->TextEx(&Cursor, m_pClient->m_aClients[pInfo->m_ClientID].m_aClan, -1);
 
-			// K
-			TextRender()->TextColor(TextColor.r, TextColor.g, TextColor.b, 0.5f*ColorAlpha);
-			str_format(aBuf, sizeof(aBuf), "%d", clamp(m_aPlayerStats[pInfo->m_ClientID].m_Kills, 0, 999));
-			tw = TextRender()->TextWidth(0, FontSize, aBuf, -1);
-			TextRender()->SetCursor(&Cursor, KillOffset+KillLength/2-tw/2, y+Spacing, FontSize, TEXTFLAG_RENDER);
-			Cursor.m_LineWidth = KillLength;
-			TextRender()->TextEx(&Cursor, aBuf, -1);
+			if(!Race)
+			{
+				// K
+				TextRender()->TextColor(TextColor.r, TextColor.g, TextColor.b, 0.5f*ColorAlpha);
+				str_format(aBuf, sizeof(aBuf), "%d", clamp(m_pClient->m_pStats->GetPlayerStats(pInfo->m_ClientID)->m_Frags, 0, 999));
+				tw = TextRender()->TextWidth(0, FontSize, aBuf, -1, -1.0f);
+				TextRender()->SetCursor(&Cursor, KillOffset+KillLength/2-tw/2, y+Spacing, FontSize, TEXTFLAG_RENDER);
+				Cursor.m_LineWidth = KillLength;
+				TextRender()->TextEx(&Cursor, aBuf, -1);
 
-			// D
-			str_format(aBuf, sizeof(aBuf), "%d", clamp(m_aPlayerStats[pInfo->m_ClientID].m_Deaths, 0, 999));
-			tw = TextRender()->TextWidth(0, FontSize, aBuf, -1);
-			TextRender()->SetCursor(&Cursor, DeathOffset+DeathLength/2-tw/2, y+Spacing, FontSize, TEXTFLAG_RENDER);
-			Cursor.m_LineWidth = DeathLength;
-			TextRender()->TextEx(&Cursor, aBuf, -1);
+				// D
+				str_format(aBuf, sizeof(aBuf), "%d", clamp(m_pClient->m_pStats->GetPlayerStats(pInfo->m_ClientID)->m_Deaths, 0, 999));
+				tw = TextRender()->TextWidth(0, FontSize, aBuf, -1, -1.0f);
+				TextRender()->SetCursor(&Cursor, DeathOffset+DeathLength/2-tw/2, y+Spacing, FontSize, TEXTFLAG_RENDER);
+				Cursor.m_LineWidth = DeathLength;
+				TextRender()->TextEx(&Cursor, aBuf, -1);
+			}
 
 			// score
+			if(Race)
+			{
+				aBuf[0] = 0;
+				if(pInfo->m_pPlayerInfo->m_Score >= 0)
+					FormatTime(aBuf, sizeof(aBuf), pInfo->m_pPlayerInfo->m_Score, m_pClient->RacePrecision());
+			}
+			else
+			{
+				str_format(aBuf, sizeof(aBuf), "%d", clamp(pInfo->m_pPlayerInfo->m_Score, -999, 9999));
+			}
+
+			tw = TextRender()->TextWidth(0, FontSize, aBuf, -1, -1.0f);
 			TextRender()->TextColor(TextColor.r, TextColor.g, TextColor.b, ColorAlpha);
-			str_format(aBuf, sizeof(aBuf), "%d", clamp(pInfo->m_pPlayerInfo->m_Score, -999, 999));
-			tw = TextRender()->TextWidth(0, FontSize, aBuf, -1);
-			TextRender()->SetCursor(&Cursor, ScoreOffset+ScoreLength/2-tw/2, y+Spacing, FontSize, TEXTFLAG_RENDER);
+			TextRender()->SetCursor(&Cursor, ScoreOffset + (Race ? ScoreLength-tw-3.f : ScoreLength/2-tw/2), y+Spacing, FontSize, TEXTFLAG_RENDER);
 			Cursor.m_LineWidth = ScoreLength;
 			TextRender()->TextEx(&Cursor, aBuf, -1);
 
@@ -599,7 +653,7 @@ float CScoreboard::RenderScoreboard(float x, float y, float w, int Team, const c
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
 	TextRender()->TextOutlineColor(0.0f, 0.0f, 0.0f, 0.3f);
 
-	return HeadlineHeight+LineHeight*(m_PlayerLines+1);
+	return HeadlineHeight+LineHeight*(PlayerLines+1);
 }
 
 void CScoreboard::RenderRecordingNotification(float x)
@@ -622,30 +676,27 @@ void CScoreboard::RenderRecordingNotification(float x)
 	char aBuf[64];
 	int Seconds = m_pClient->DemoRecorder()->Length();
 	str_format(aBuf, sizeof(aBuf), Localize("REC %3d:%02d"), Seconds/60, Seconds%60);
-	TextRender()->Text(0, x+50.0f, 10.0f, 20.0f, aBuf, -1);
+	TextRender()->Text(0, x+50.0f, 10.0f, 20.0f, aBuf, -1.0f);
 }
 
 void CScoreboard::OnRender()
 {
-	// check if we need to reset the player stats
-	if(!m_SkipPlayerStatsReset && m_pClient->m_Snap.m_pGameData && m_pClient->m_Snap.m_pGameData->m_GameStartTick == Client()->GameTick())
+	// don't render scoreboard if menu or statboard is open
+	if(m_pClient->m_pMenus->IsActive() || m_pClient->m_pStats->IsActive())
+		return;
+
+	// postpone the active state till the render area gets updated during the rendering
+	if(m_Activate)
 	{
-		m_SkipPlayerStatsReset = true;
-		for(int i = 0; i < MAX_CLIENTS; i++)
-			ResetPlayerStats(i);
+		m_Active = true;
+		m_Activate = false;
 	}
-	else if(m_SkipPlayerStatsReset && m_pClient->m_Snap.m_pGameData && m_pClient->m_Snap.m_pGameData->m_GameStartTick != Client()->GameTick())
-		m_SkipPlayerStatsReset = false;
 
 	// close the motd if we actively wanna look on the scoreboard
 	if(m_Active)
 		m_pClient->m_pMotd->Clear();
 
-	if(!Active())
-		return;
-
-	// don't render scoreboard if menu or motd is open
-	if(m_pClient->m_pMenus->IsActive() || m_pClient->m_pMotd->IsActive())
+	if(!IsActive())
 		return;
 
 	CUIRect Screen = *UI()->Screen();
@@ -707,45 +758,23 @@ void CScoreboard::OnRender()
 			else
 				str_copy(aText, Localize("Draw!"), sizeof(aText));
 
-			float tw = TextRender()->TextWidth(0, FontSize, aText, -1);
-			TextRender()->Text(0, Width/2-tw/2, 39, FontSize, aText, -1);
+			float tw = TextRender()->TextWidth(0, FontSize, aText, -1, -1.0f);
+			TextRender()->Text(0, Width/2-tw/2, 39, FontSize, aText, -1.0f);
 		}
 		else if(m_pClient->m_Snap.m_pGameData->m_GameStateFlags&GAMESTATEFLAG_ROUNDOVER)
 		{
 			char aText[256];
 			str_copy(aText, Localize("Round over!"), sizeof(aText));
 
-			float tw = TextRender()->TextWidth(0, FontSize, aText, -1);
-			TextRender()->Text(0, Width/2-tw/2, 39, FontSize, aText, -1);
-			m_SkipPlayerStatsReset = true;
+			float tw = TextRender()->TextWidth(0, FontSize, aText, -1, -1.0f);
+			TextRender()->Text(0, Width/2-tw/2, 39, FontSize, aText, -1.0f);
 		}
 	}
 
 	RenderRecordingNotification((Width/7)*4);
 }
 
-void CScoreboard::OnMessage(int MsgType, void *pRawMsg)
-{
-	if(MsgType == NETMSGTYPE_SV_KILLMSG)
-	{
-		CNetMsg_Sv_KillMsg *pMsg = (CNetMsg_Sv_KillMsg *)pRawMsg;
-
-		m_aPlayerStats[pMsg->m_Victim].m_Deaths++;
-		if(pMsg->m_Victim != pMsg->m_Killer)
-			m_aPlayerStats[pMsg->m_Killer].m_Kills++;
-	}
-	else if(MsgType == NETMSGTYPE_SV_CLIENTINFO && Client()->State() != IClient::STATE_DEMOPLAYBACK)
-	{
-		CNetMsg_Sv_ClientInfo *pMsg = (CNetMsg_Sv_ClientInfo *)pRawMsg;
-
-		if(!m_pClient->m_aClients[pMsg->m_ClientID].m_Active)
-			return;
-
-		ResetPlayerStats(pMsg->m_ClientID);
-	}
-}
-
-bool CScoreboard::Active()
+bool CScoreboard::IsActive() const
 {
 	// if we actively wanna look on the scoreboard
 	if(m_Active)
@@ -766,16 +795,6 @@ bool CScoreboard::Active()
 	return false;
 }
 
-void CScoreboard::ResetPlayerStats(int ClientID)
-{
-	m_aPlayerStats[ClientID].Reset();
-}
-
-CUIRect CScoreboard::GetScoreboardRect()
-{
-	return m_TotalRect;
-}
-
 const char *CScoreboard::GetClanName(int Team)
 {
 	int ClanPlayers = 0;
@@ -804,15 +823,3 @@ const char *CScoreboard::GetClanName(int Team)
 	else
 		return 0;
 }
-
-CScoreboard::CPlayerStats::CPlayerStats()
-{
-	m_Kills = 0;
-	m_Deaths = 0;
-}
-
-void CScoreboard::CPlayerStats::Reset()
-{
-	m_Kills = 0;
-	m_Deaths = 0;
-}
