@@ -10,29 +10,28 @@
 #include <engine/storage.h>
 #include <engine/external/json-parser/json.h>
 #include <engine/shared/config.h>
+#include <engine/shared/jsonwriter.h>
 
+#include "menus.h"
 #include "skins.h"
 
 
 const char * const CSkins::ms_apSkinPartNames[NUM_SKINPARTS] = {"body", "marking", "decoration", "hands", "feet", "eyes"}; /* Localize("body","skins");Localize("marking","skins");Localize("decoration","skins");Localize("hands","skins");Localize("feet","skins");Localize("eyes","skins"); */
 const char * const CSkins::ms_apColorComponents[NUM_COLOR_COMPONENTS] = {"hue", "sat", "lgt", "alp"};
 
-char *const CSkins::ms_apSkinVariables[NUM_SKINPARTS] = {g_Config.m_PlayerSkinBody, g_Config.m_PlayerSkinMarking, g_Config.m_PlayerSkinDecoration,
-													g_Config.m_PlayerSkinHands, g_Config.m_PlayerSkinFeet, g_Config.m_PlayerSkinEyes};
-int *const CSkins::ms_apUCCVariables[NUM_SKINPARTS] = {&g_Config.m_PlayerUseCustomColorBody, &g_Config.m_PlayerUseCustomColorMarking, &g_Config.m_PlayerUseCustomColorDecoration,
-													&g_Config.m_PlayerUseCustomColorHands, &g_Config.m_PlayerUseCustomColorFeet, &g_Config.m_PlayerUseCustomColorEyes};
-int *const CSkins::ms_apColorVariables[NUM_SKINPARTS] = {&g_Config.m_PlayerColorBody, &g_Config.m_PlayerColorMarking, &g_Config.m_PlayerColorDecoration,
-													&g_Config.m_PlayerColorHands, &g_Config.m_PlayerColorFeet, &g_Config.m_PlayerColorEyes};
+char *CSkins::ms_apSkinVariables[NUM_SKINPARTS] = {0};
+int *CSkins::ms_apUCCVariables[NUM_SKINPARTS] = {0};
+int *CSkins::ms_apColorVariables[NUM_SKINPARTS] = {0};
 
+const float MIN_EYE_BODY_COLOR_DIST = 80.f; // between body and eyes (LAB color space)
 
 int CSkins::SkinPartScan(const char *pName, int IsDir, int DirType, void *pUser)
 {
 	CSkins *pSelf = (CSkins *)pUser;
-	int l = str_length(pName);
-	if(l < 4 || IsDir || str_comp(pName+l-4, ".png") != 0)
+	if(IsDir || !str_endswith(pName, ".png"))
 		return 0;
 
-	char aBuf[512];
+	char aBuf[IO_MAX_PATH_LENGTH];
 	str_format(aBuf, sizeof(aBuf), "skins/%s/%s", CSkins::ms_apSkinPartNames[pSelf->m_ScanningPart], pName);
 	CImageInfo Info;
 	if(!pSelf->Graphics()->LoadPNG(&Info, aBuf, DirType))
@@ -93,8 +92,8 @@ int CSkins::SkinPartScan(const char *pName, int IsDir, int DirType, void *pUser)
 		Part.m_Flags |= SKINFLAG_SPECIAL;
 	if(DirType != IStorage::TYPE_SAVE)
 		Part.m_Flags |= SKINFLAG_STANDARD;
-	str_copy(Part.m_aName, pName, min((int)sizeof(Part.m_aName),l-3));
-	if(g_Config.m_Debug)
+	str_truncate(Part.m_aName, sizeof(Part.m_aName), pName, str_length(pName) - 4);
+	if(pSelf->Config()->m_Debug)
 	{
 		str_format(aBuf, sizeof(aBuf), "load skin part %s", Part.m_aName);
 		pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "skins", aBuf);
@@ -106,14 +105,13 @@ int CSkins::SkinPartScan(const char *pName, int IsDir, int DirType, void *pUser)
 
 int CSkins::SkinScan(const char *pName, int IsDir, int DirType, void *pUser)
 {
-	int l = str_length(pName);
-	if(l < 5 || IsDir || str_comp(pName+l-5, ".json") != 0)
+	if(IsDir || !str_endswith(pName, ".json"))
 		return 0;
 
 	CSkins *pSelf = (CSkins *)pUser;
 
 	// read file data into buffer
-	char aBuf[512];
+	char aBuf[IO_MAX_PATH_LENGTH];
 	str_format(aBuf, sizeof(aBuf), "skins/%s", pName);
 	IOHANDLE File = pSelf->Storage()->OpenFile(aBuf, IOFLAG_READ, IStorage::TYPE_ALL);
 	if(!File)
@@ -125,7 +123,7 @@ int CSkins::SkinScan(const char *pName, int IsDir, int DirType, void *pUser)
 
 	// init
 	CSkin Skin = pSelf->m_DummySkin;
-	str_copy(Skin.m_aName, pName, min((int)sizeof(Skin.m_aName),l-4));
+	str_truncate(Skin.m_aName, sizeof(Skin.m_aName), pName, str_length(pName) - 5);
 	if(pSelf->Find(Skin.m_aName, true) != -1)
 		return 0;
 	bool SpecialSkin = pName[0] == 'x' && pName[1] == '_';
@@ -166,9 +164,9 @@ int CSkins::SkinScan(const char *pName, int IsDir, int DirType, void *pUser)
 			bool UseCustomColors = false;
 			const json_value &rColour = rPart["custom_colors"];
 			if(rColour.type == json_string)
-			{
 				UseCustomColors = str_comp((const char *)rColour, "true") == 0;
-			}
+			else if(rColour.type == json_boolean)
+				UseCustomColors = rColour.u.boolean;
 			Skin.m_aUseCustomColors[PartIndex] = UseCustomColors;
 
 			// color components
@@ -202,7 +200,7 @@ int CSkins::SkinScan(const char *pName, int IsDir, int DirType, void *pUser)
 	Skin.m_Flags = SpecialSkin ? SKINFLAG_SPECIAL : 0;
 	if(DirType != IStorage::TYPE_SAVE)
 		Skin.m_Flags |= SKINFLAG_STANDARD;
-	if(g_Config.m_Debug)
+	if(pSelf->Config()->m_Debug)
 	{
 		str_format(aBuf, sizeof(aBuf), "load skin %s", Skin.m_aName);
 		pSelf->Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "skins", aBuf);
@@ -212,9 +210,32 @@ int CSkins::SkinScan(const char *pName, int IsDir, int DirType, void *pUser)
 	return 0;
 }
 
+int CSkins::GetInitAmount() const
+{
+	return NUM_SKINPARTS*2 + 8;
+}
 
 void CSkins::OnInit()
 {
+	ms_apSkinVariables[SKINPART_BODY] = Config()->m_PlayerSkinBody;
+	ms_apSkinVariables[SKINPART_MARKING] = Config()->m_PlayerSkinMarking;
+	ms_apSkinVariables[SKINPART_DECORATION] = Config()->m_PlayerSkinDecoration;
+	ms_apSkinVariables[SKINPART_HANDS] = Config()->m_PlayerSkinHands;
+	ms_apSkinVariables[SKINPART_FEET] = Config()->m_PlayerSkinFeet;
+	ms_apSkinVariables[SKINPART_EYES] = Config()->m_PlayerSkinEyes;
+	ms_apUCCVariables[SKINPART_BODY] = &Config()->m_PlayerUseCustomColorBody;
+	ms_apUCCVariables[SKINPART_MARKING] = &Config()->m_PlayerUseCustomColorMarking;
+	ms_apUCCVariables[SKINPART_DECORATION] = &Config()->m_PlayerUseCustomColorDecoration;
+	ms_apUCCVariables[SKINPART_HANDS] = &Config()->m_PlayerUseCustomColorHands;
+	ms_apUCCVariables[SKINPART_FEET] = &Config()->m_PlayerUseCustomColorFeet;
+	ms_apUCCVariables[SKINPART_EYES] = &Config()->m_PlayerUseCustomColorEyes;
+	ms_apColorVariables[SKINPART_BODY] = &Config()->m_PlayerColorBody;
+	ms_apColorVariables[SKINPART_MARKING] = &Config()->m_PlayerColorMarking;
+	ms_apColorVariables[SKINPART_DECORATION] = &Config()->m_PlayerColorDecoration;
+	ms_apColorVariables[SKINPART_HANDS] = &Config()->m_PlayerColorHands;
+	ms_apColorVariables[SKINPART_FEET] = &Config()->m_PlayerColorFeet;
+	ms_apColorVariables[SKINPART_EYES] = &Config()->m_PlayerColorEyes;
+
 	for(int p = 0; p < NUM_SKINPARTS; p++)
 	{
 		m_aaSkinParts[p].clear();
@@ -244,6 +265,7 @@ void CSkins::OnInit()
 			DummySkinPart.m_BloodColor = vec3(1.0f, 1.0f, 1.0f);
 			m_aaSkinParts[p].add(DummySkinPart);
 		}
+		m_pClient->m_pMenus->RenderLoading(2);
 	}
 
 	// create dummy skin
@@ -262,31 +284,56 @@ void CSkins::OnInit()
 		m_DummySkin.m_aPartColors[p] = p==SKINPART_MARKING ? (255<<24)+65408 : 65408;
 		m_DummySkin.m_aUseCustomColors[p] = 0;
 	}
+	m_pClient->m_pMenus->RenderLoading(1);
 
 	// load skins
 	m_aSkins.clear();
 	Storage()->ListDirectory(IStorage::TYPE_ALL, "skins", SkinScan, this);
+	m_pClient->m_pMenus->RenderLoading(5);
 
 	// add dummy skin
 	if(!m_aSkins.size())
 		m_aSkins.add(m_DummySkin);
 
-	// add xmas hat
-	const char *pFileName = "skins/xmas_hat.png";
-	CImageInfo Info;
-	if(!Graphics()->LoadPNG(&Info, pFileName, IStorage::TYPE_ALL) || Info.m_Width != 128 || Info.m_Height != 512)
 	{
-		char aBuf[128];
-		str_format(aBuf, sizeof(aBuf), "failed to load xmas hat '%s'", pFileName);
-		Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
+		// add xmas hat
+		const char *pFileName = "skins/xmas_hat.png";
+		CImageInfo Info;
+		if(!Graphics()->LoadPNG(&Info, pFileName, IStorage::TYPE_ALL) || Info.m_Width != 128 || Info.m_Height != 512)
+		{
+			char aBuf[128];
+			str_format(aBuf, sizeof(aBuf), "failed to load xmas hat '%s'", pFileName);
+			Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
+		}
+		else
+		{
+			char aBuf[128];
+			str_format(aBuf, sizeof(aBuf), "loaded xmas hat '%s'", pFileName);
+			Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
+			m_XmasHatTexture = Graphics()->LoadTextureRaw(Info.m_Width, Info.m_Height, Info.m_Format, Info.m_pData, Info.m_Format, 0);
+		}
 	}
-	else
+	m_pClient->m_pMenus->RenderLoading(1);
+
 	{
-		char aBuf[128];
-		str_format(aBuf, sizeof(aBuf), "loaded xmas hat '%s'", pFileName);
-		Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
-		m_XmasHatTexture = Graphics()->LoadTextureRaw(Info.m_Width, Info.m_Height, Info.m_Format, Info.m_pData, Info.m_Format, 0);
+		// add bot decoration
+		const char *pFileName = "skins/bot.png";
+		CImageInfo Info;
+		if(!Graphics()->LoadPNG(&Info, pFileName, IStorage::TYPE_ALL) || Info.m_Width != 384 || Info.m_Height != 160)
+		{
+			char aBuf[128];
+			str_format(aBuf, sizeof(aBuf), "failed to load bot '%s'", pFileName);
+			Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
+		}
+		else
+		{
+			char aBuf[128];
+			str_format(aBuf, sizeof(aBuf), "loaded bot '%s'", pFileName);
+			Console()->Print(IConsole::OUTPUT_LEVEL_ADDINFO, "game", aBuf);
+			m_BotTexture = Graphics()->LoadTextureRaw(Info.m_Width, Info.m_Height, Info.m_Format, Info.m_pData, Info.m_Format, 0);
+		}
 	}
+	m_pClient->m_pMenus->RenderLoading(1);
 }
 
 void CSkins::AddSkin(const char *pSkinName)
@@ -397,3 +444,140 @@ int CSkins::GetTeamColor(int UseCustomColors, int PartColor, int Team, int Part)
 
 	return ColorVal;
 }
+
+bool CSkins::ValidateSkinParts(char* aPartNames[NUM_SKINPARTS], int* aUseCustomColors, int* aPartColors, int GameFlags) const
+{
+	// force standard (black) eyes on team skins
+	if(GameFlags&GAMEFLAG_TEAMS)
+	{
+		// TODO: adjust eye color here as well?
+		if(str_comp(aPartNames[SKINPART_EYES], "colorable") == 0 || str_comp(aPartNames[SKINPART_EYES], "negative") == 0)
+		{
+			str_copy(aPartNames[SKINPART_EYES], "standard", 24);
+			return false;
+		}
+	}
+	else
+	{
+		const int BodyColor = aPartColors[SKINPART_BODY];
+		const int EyeColor = aPartColors[SKINPART_EYES];
+
+		vec3 BodyHsl(((BodyColor>>16)&0xff)/255.0f, ((BodyColor>>8)&0xff)/255.0f, (BodyColor&0xff)/255.0f);
+		vec3 EyeHsl(((EyeColor>>16)&0xff)/255.0f, ((EyeColor>>8)&0xff)/255.0f, (EyeColor&0xff)/255.0f);
+
+		if(!aUseCustomColors[SKINPART_BODY])
+			BodyHsl = vec3(0, 0, 1);
+
+		vec3 BodyRgb = HslToRgb(BodyHsl);
+		vec3 BodyLab = RgbToLab(BodyRgb);
+
+		if(str_comp(aPartNames[SKINPART_EYES], "negative") == 0)
+		{
+			if(!aUseCustomColors[SKINPART_EYES])
+				EyeHsl = vec3(0, 0, 1);
+
+			vec3 OrgEyeHsl = EyeHsl;
+			EyeHsl.l *= 0.925f;
+
+			vec3 EyeRgb = HslToRgb(EyeHsl);
+			vec3 EyeLab = RgbToLab(EyeRgb);
+			float Dist = LabDistance(BodyLab, EyeLab);
+
+			if(Dist < MIN_EYE_BODY_COLOR_DIST)
+			{
+				OrgEyeHsl.l = clamp(OrgEyeHsl.l - 0.22f, 0.f, 1.f);
+
+				// white eye can't go to black because of our DARKEST_COLOR_LGT restriction, so switch to standard (black) eyes
+				if(OrgEyeHsl.l < DARKEST_COLOR_LGT/255.f)
+					str_copy(aPartNames[SKINPART_EYES], "standard", 24); // black
+				else
+				{
+					aUseCustomColors[SKINPART_EYES] = 1;
+					aPartColors[SKINPART_EYES] = (int(OrgEyeHsl.h*255) << 16) | (int(OrgEyeHsl.s*255) << 8) | (int(OrgEyeHsl.l*255));
+				}
+
+				return false;
+			}
+		}
+		else if(str_comp(aPartNames[SKINPART_EYES], "colorable") == 0)
+		{
+			if(!aUseCustomColors[SKINPART_EYES])
+				EyeHsl = vec3(0, 0, 1);
+
+			vec3 OrgEyeHsl = EyeHsl;
+			EyeHsl.l = clamp(EyeHsl.l * 0.0823f, 0.f, 1.f);
+
+
+			vec3 EyeRgb = HslToRgb(EyeHsl);
+			vec3 EyeLab = RgbToLab(EyeRgb);
+			float Dist = LabDistance(BodyLab, EyeLab);
+
+			if(Dist < MIN_EYE_BODY_COLOR_DIST)
+			{
+				OrgEyeHsl.l -= 0.6f;
+				OrgEyeHsl.l = clamp(OrgEyeHsl.l, 0.f, 1.f);
+
+				aUseCustomColors[SKINPART_EYES] = 1;
+				aPartColors[SKINPART_EYES] = (int(OrgEyeHsl.h*255) << 16) | (int(OrgEyeHsl.s*255) << 8) | (int(OrgEyeHsl.l*255));
+
+				return false;
+			}
+		}
+	}
+
+	return true;
+}
+
+void CSkins::SaveSkinfile(const char *pSaveSkinName)
+{
+	char aBuf[IO_MAX_PATH_LENGTH];
+	str_format(aBuf, sizeof(aBuf), "skins/%s.json", pSaveSkinName);
+	IOHANDLE File = Storage()->OpenFile(aBuf, IOFLAG_WRITE, IStorage::TYPE_SAVE);
+	if(!File)
+		return;
+
+	CJsonWriter Writer(File);
+
+	Writer.BeginObject();
+	Writer.WriteAttribute("skin");
+	Writer.BeginObject();
+	for(int PartIndex = 0; PartIndex < NUM_SKINPARTS; PartIndex++)
+	{
+		if(!ms_apSkinVariables[PartIndex][0])
+			continue;
+
+		// part start
+		Writer.WriteAttribute(ms_apSkinPartNames[PartIndex]);
+		Writer.BeginObject();
+		{
+			Writer.WriteAttribute("filename");
+			Writer.WriteStrValue(ms_apSkinVariables[PartIndex]);
+
+			const bool CustomColors = *ms_apUCCVariables[PartIndex];
+			Writer.WriteAttribute("custom_colors");
+			Writer.WriteBoolValue(CustomColors);
+
+			if(CustomColors)
+			{
+				for(int c = 0; c < NUM_COLOR_COMPONENTS-1; c++)
+				{
+					int Val = (*ms_apColorVariables[PartIndex] >> (2-c)*8) & 0xff;
+					Writer.WriteAttribute(ms_apColorComponents[c]);
+					Writer.WriteIntValue(Val);
+				}
+				if(PartIndex == SKINPART_MARKING)
+				{
+					int Val = (*ms_apColorVariables[PartIndex] >> 24) & 0xff;
+					Writer.WriteAttribute(ms_apColorComponents[3]);
+					Writer.WriteIntValue(Val);
+				}
+			}
+		}
+		Writer.EndObject();
+	}
+	Writer.EndObject();
+	Writer.EndObject();
+
+	// add new skin to the skin list
+	AddSkin(pSaveSkinName);
+}
