@@ -49,12 +49,13 @@ public:
 	static const char * const ms_apSkinPartNames[NUM_SKINPARTS];
 	static const char * const ms_apColorComponents[NUM_COLOR_COMPONENTS];
 
-	static char * const ms_apSkinVariables[NUM_SKINPARTS];
-	static int * const ms_apUCCVariables[NUM_SKINPARTS]; // use custom color
-	static int * const ms_apColorVariables[NUM_SKINPARTS];
+	static char *ms_apSkinVariables[NUM_SKINPARTS];
+	static int *ms_apUCCVariables[NUM_SKINPARTS]; // use custom color
+	static int *ms_apColorVariables[NUM_SKINPARTS];
 	IGraphics::CTextureHandle m_XmasHatTexture;
+	IGraphics::CTextureHandle m_BotTexture;
 
-	//
+	int GetInitAmount() const;
 	void OnInit();
 
 	void AddSkin(const char *pSkinName);
@@ -71,6 +72,11 @@ public:
 	vec4 GetColorV4(int v, bool UseAlpha) const;
 	int GetTeamColor(int UseCustomColors, int PartColor, int Team, int Part) const;
 
+	// returns true if everything was valid and nothing changed
+	bool ValidateSkinParts(char *aPartNames[NUM_SKINPARTS], int *aUseCustomColors, int* aPartColors, int GameFlags) const;
+
+	void SaveSkinfile(const char *pSaveSkinName);
+
 private:
 	int m_ScanningPart;
 	sorted_array<CSkinPart> m_aaSkinParts[NUM_SKINPARTS];
