@@ -30,150 +30,89 @@ void CSpectator::ConSpectate(IConsole::IResult *pResult, void *pUserData)
 		pSelf->Spectate(pResult->GetInteger(0), pResult->GetInteger(1));
 }
 
-void CSpectator::ConSpectateNext(IConsole::IResult *pResult, void *pUserData)
+bool CSpectator::SpecModePossible(int SpecMode, int SpectatorID)
 {
-	CSpectator *pSelf = (CSpectator *)pUserData;
-	if(!pSelf->m_pClient->m_Snap.m_SpecInfo.m_Active || (pSelf->Client()->State() == IClient::STATE_DEMOPLAYBACK && pSelf->DemoPlayer()->GetDemoType() != IDemoPlayer::DEMOTYPE_SERVER))
-		return;
-
-	int NewSpecMode = pSelf->m_pClient->m_Snap.m_SpecInfo.m_SpecMode;
-	int NewSpectatorID = -1;
-	bool GotNewSpectatorID = false;
-
-	if(NewSpecMode != SPEC_PLAYER)
-		NewSpecMode = (NewSpecMode + 1) % NUM_SPECMODES;
-	else
-		NewSpectatorID = pSelf->m_pClient->m_Snap.m_SpecInfo.m_SpectatorID;
-
-	switch(NewSpecMode)
-	{	// drop through
+	int i = SpectatorID;
+	switch(SpecMode)
+	{
 	case SPEC_PLAYER:
-		for(int i = NewSpectatorID + 1; i < MAX_CLIENTS; i++)
+		if(!m_pClient->m_aClients[i].m_Active || m_pClient->m_aClients[i].m_Team == TEAM_SPECTATORS)
 		{
-			if(!pSelf->m_pClient->m_aClients[i].m_Active || pSelf->m_pClient->m_aClients[i].m_Team == TEAM_SPECTATORS)
-				continue;
-
-			if(pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS &&
-				(i == pSelf->m_pClient->m_LocalClientID || pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != pSelf->m_pClient->m_aClients[i].m_Team ||
-				(pSelf->m_pClient->m_Snap.m_paPlayerInfos[i] && (pSelf->m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD))))
-				continue;
-
-			NewSpectatorID = i;
-			GotNewSpectatorID = true;
-			break;
+			return false;
 		}
-		if(GotNewSpectatorID)
-			break;
-		NewSpecMode = SPEC_FLAGRED;
-		NewSpectatorID = -1;
+		if(m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS &&
+			(i == m_pClient->m_LocalClientID || m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team != m_pClient->m_aClients[i].m_Team ||
+			(m_pClient->m_Snap.m_paPlayerInfos[i] && (m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD))))
+		{
+			return false;
+		}
+		return true;
 	case SPEC_FLAGRED:
 	case SPEC_FLAGBLUE:
-		if(pSelf->m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_FLAGS)
-		{
-			GotNewSpectatorID = true;
-			break;
-		}
-		NewSpecMode = SPEC_FREEVIEW;
+		return m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_FLAGS;
 	case SPEC_FREEVIEW:
-		if(pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team == TEAM_SPECTATORS)
-			GotNewSpectatorID = true;
-		else
+		return m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team == TEAM_SPECTATORS;
+	default:
+		dbg_assert(false, "invalid spec mode");
+		return false;
+	}
+}
+
+static void IterateSpecMode(int Direction, int *pSpecMode, int *pSpectatorID)
+{
+	dbg_assert(Direction == -1 || Direction == 1, "invalid direction");
+	if(*pSpecMode == SPEC_PLAYER)
+	{
+		*pSpectatorID += Direction;
+		if(0 <= *pSpectatorID && *pSpectatorID < MAX_CLIENTS)
 		{
-			// start again on the beginning in survival
-			for(int i = 0; i < pSelf->m_pClient->m_Snap.m_SpecInfo.m_SpectatorID; i++)
-			{
-				if(!pSelf->m_pClient->m_aClients[i].m_Active || pSelf->m_pClient->m_aClients[i].m_Team == TEAM_SPECTATORS)
-					continue;
-
-				if(pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS &&
-					(i == pSelf->m_pClient->m_LocalClientID || pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != pSelf->m_pClient->m_aClients[i].m_Team ||
-					(pSelf->m_pClient->m_Snap.m_paPlayerInfos[i] && (pSelf->m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD))))
-					continue;
-
-				NewSpecMode = SPEC_PLAYER;
-				NewSpectatorID = i;
-				GotNewSpectatorID = true;
-				break;
-			}
+			return;
+		}
+		*pSpectatorID = -1;
+	}
+	*pSpecMode = (*pSpecMode + Direction + NUM_SPECMODES) % NUM_SPECMODES;
+	if(*pSpecMode == SPEC_PLAYER)
+	{
+		*pSpectatorID = 0;
+		if(Direction == -1)
+		{
+			*pSpectatorID = MAX_CLIENTS - 1;
 		}
 	}
+}
 
-	if(GotNewSpectatorID)
-		pSelf->Spectate(NewSpecMode, NewSpectatorID);
+void CSpectator::HandleSpectateNextPrev(int Direction)
+{
+	if(!m_pClient->m_Snap.m_SpecInfo.m_Active || (Client()->State() == IClient::STATE_DEMOPLAYBACK && DemoPlayer()->GetDemoType() != IDemoPlayer::DEMOTYPE_SERVER))
+		return;
+
+	int NewSpecMode = m_pClient->m_Snap.m_SpecInfo.m_SpecMode;
+	int NewSpectatorID = -1;
+	if(NewSpecMode == SPEC_PLAYER)
+	{
+		NewSpectatorID = m_pClient->m_Snap.m_SpecInfo.m_SpectatorID;
+	}
+
+	// Ensure the loop terminates even if no spec modes are possible.
+	for(int i = 0; i < NUM_SPECMODES + MAX_CLIENTS; i++)
+	{
+		IterateSpecMode(Direction, &NewSpecMode, &NewSpectatorID);
+		if(SpecModePossible(NewSpecMode, NewSpectatorID))
+		{
+			Spectate(NewSpecMode, NewSpectatorID);
+			return;
+		}
+	}
+}
+
+void CSpectator::ConSpectateNext(IConsole::IResult *pResult, void *pUserData)
+{
+	((CSpectator *)pUserData)->HandleSpectateNextPrev(1);
 }
 
 void CSpectator::ConSpectatePrevious(IConsole::IResult *pResult, void *pUserData)
 {
-	CSpectator *pSelf = (CSpectator *)pUserData;
-	if(!pSelf->m_pClient->m_Snap.m_SpecInfo.m_Active || (pSelf->Client()->State() == IClient::STATE_DEMOPLAYBACK && pSelf->DemoPlayer()->GetDemoType() != IDemoPlayer::DEMOTYPE_SERVER))
-		return;
-
-	int NewSpecMode = pSelf->m_pClient->m_Snap.m_SpecInfo.m_SpecMode;
-	int NewSpectatorID = MAX_CLIENTS;
-	bool GotNewSpectatorID = false;
-
-	if(NewSpecMode != SPEC_PLAYER)
-		NewSpecMode = (NewSpecMode - 1 + NUM_SPECMODES) % NUM_SPECMODES;
-	else
-		NewSpectatorID = pSelf->m_pClient->m_Snap.m_SpecInfo.m_SpectatorID;
-
-	switch(NewSpecMode)
-	{	// drop through
-	case SPEC_FLAGBLUE:
-	case SPEC_FLAGRED:
-		if(pSelf->m_pClient->m_GameInfo.m_GameFlags&GAMEFLAG_FLAGS)
-		{
-			NewSpectatorID = -1;
-			GotNewSpectatorID = true;
-			break;
-		}
-		NewSpecMode = SPEC_PLAYER;
-		NewSpectatorID = MAX_CLIENTS;
-	case SPEC_PLAYER:
-		for(int i = NewSpectatorID - 1; i >= 0; i--)
-		{
-			if(!pSelf->m_pClient->m_aClients[i].m_Active || pSelf->m_pClient->m_aClients[i].m_Team == TEAM_SPECTATORS)
-				continue;
-
-			if(pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS &&
-				(i == pSelf->m_pClient->m_LocalClientID || pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != pSelf->m_pClient->m_aClients[i].m_Team ||
-				(pSelf->m_pClient->m_Snap.m_paPlayerInfos[i] && (pSelf->m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD))))
-				continue;
-
-			NewSpectatorID = i;
-			GotNewSpectatorID = true;
-			break;
-		}
-		if(GotNewSpectatorID)
-			break;
-		NewSpecMode = SPEC_FREEVIEW;
-	case SPEC_FREEVIEW:
-		NewSpectatorID = -1;
-		if(pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team == TEAM_SPECTATORS)
-			GotNewSpectatorID = true;
-		else
-		{
-			// start again on the beginning in survival
-			for(int i = MAX_CLIENTS-1; i > pSelf->m_pClient->m_Snap.m_SpecInfo.m_SpectatorID; i--)
-			{
-				if(!pSelf->m_pClient->m_aClients[i].m_Active || pSelf->m_pClient->m_aClients[i].m_Team == TEAM_SPECTATORS)
-					continue;
-
-				if(pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS &&
-					(i == pSelf->m_pClient->m_LocalClientID || pSelf->m_pClient->m_aClients[pSelf->m_pClient->m_LocalClientID].m_Team != pSelf->m_pClient->m_aClients[i].m_Team ||
-					(pSelf->m_pClient->m_Snap.m_paPlayerInfos[i] && (pSelf->m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD))))
-					continue;
-
-				NewSpecMode = SPEC_PLAYER;
-				NewSpectatorID = i;
-				GotNewSpectatorID = true;
-				break;
-			}
-		}
-	}
-
-	if(GotNewSpectatorID)
-		pSelf->Spectate(NewSpecMode, NewSpectatorID);
+	((CSpectator *)pUserData)->HandleSpectateNextPrev(-1);
 }
 
 CSpectator::CSpectator()
@@ -184,7 +123,7 @@ CSpectator::CSpectator()
 void CSpectator::OnConsoleInit()
 {
 	Console()->Register("+spectate", "", CFGFLAG_CLIENT, ConKeySpectator, this, "Open spectator mode selector");
-	Console()->Register("spectate", "ii", CFGFLAG_CLIENT, ConSpectate, this, "Switch spectator mode");
+	Console()->Register("spectate", "i[mode] i[target]", CFGFLAG_CLIENT, ConSpectate, this, "Switch spectator mode");
 	Console()->Register("spectate_next", "", CFGFLAG_CLIENT, ConSpectateNext, this, "Spectate the next player");
 	Console()->Register("spectate_previous", "", CFGFLAG_CLIENT, ConSpectatePrevious, this, "Spectate the previous player");
 }
@@ -289,7 +228,7 @@ void CSpectator::OnRender()
 			Selected = true;
 		}
 		TextRender()->TextColor(1.0f, 1.0f, 1.0f, Selected?1.0f:0.5f);
-		TextRender()->Text(0, Width/2.0f-240.0f, Height/2.0f-265.0f, FontSize, Localize("Free-View"), -1);
+		TextRender()->Text(0, Width/2.0f-240.0f, Height/2.0f-265.0f, FontSize, Localize("Free-View"), -1.0f);
 	}
 
 	//
@@ -321,7 +260,7 @@ void CSpectator::OnRender()
 
 			RenderTools()->SelectSprite(Flag == SPEC_FLAGRED ? SPRITE_FLAG_RED : SPRITE_FLAG_BLUE);
 
-			float Size = LineHeight/1.5f + (Selected ? 12.0f : 8.0f);
+			float Size = 60.0f/1.5f + (Selected ? 12.0f : 8.0f);
 			float FlagWidth = Width/2.0f + x + 40.0f + (Selected ? -3.0f : -2.0f);
 			float FlagHeight = Height/2.0f + y + (Selected ? -6.0f : -4.0f);
 
@@ -338,8 +277,9 @@ void CSpectator::OnRender()
 	for(int i = 0, Count = 0; i < MAX_CLIENTS; ++i)
 	{
 		if(!m_pClient->m_Snap.m_paPlayerInfos[i] || m_pClient->m_aClients[i].m_Team == TEAM_SPECTATORS ||
-			(m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS && (m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD ||
-			m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team != m_pClient->m_aClients[i].m_Team || i == m_pClient->m_LocalClientID)))
+			m_pClient->m_Snap.m_paPlayerInfos[i]->m_PlayerFlags&PLAYERFLAG_DEAD ||
+			(m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team != TEAM_SPECTATORS &&
+			(m_pClient->m_aClients[m_pClient->m_LocalClientID].m_Team != m_pClient->m_aClients[i].m_Team || i == m_pClient->m_LocalClientID)))
 			continue;
 
 		if(Count != 0 && Count%ColumnSize == 0)
@@ -368,7 +308,7 @@ void CSpectator::OnRender()
 		}
 		TextRender()->TextColor(1.0f, 1.0f, 1.0f, Selected?1.0f:0.5f);
 		char aBuf[64];
-		str_format(aBuf, sizeof(aBuf), "%s", g_Config.m_ClShowsocial ? m_pClient->m_aClients[i].m_aName : "");
+		str_format(aBuf, sizeof(aBuf), "%s", Config()->m_ClShowsocial ? m_pClient->m_aClients[i].m_aName : "");
 
 		CTextCursor Cursor;
 		TextRender()->SetCursor(&Cursor, Width/2.0f+x+50.0f, Height/2.0f+y+5.0f, FontSize, TEXTFLAG_RENDER);
@@ -385,10 +325,10 @@ void CSpectator::OnRender()
 			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_GAME].m_Id);
 			Graphics()->QuadsBegin();
 
-			RenderTools()->SelectSprite(m_pClient->m_aClients[i].m_Team==TEAM_RED ? SPRITE_FLAG_BLUE : SPRITE_FLAG_RED, SPRITE_FLAG_FLIP_X);
+			RenderTools()->SelectSprite(i == m_pClient->m_Snap.m_pGameDataFlag->m_FlagCarrierBlue ? SPRITE_FLAG_BLUE : SPRITE_FLAG_RED, SPRITE_FLAG_FLIP_X);
 
 			float Size = LineHeight;
-			IGraphics::CQuadItem QuadItem(Width/2.0f+x-Size/5.0f, Height/2.0f+y-Size/3.0f, Size/2.0f, Size);
+			IGraphics::CQuadItem QuadItem(Width/2.0f+x+20.0f-Size/4.0f, Height/2.0f+y+20.0f-Size/1.5f, Size/2.0f, Size);
 			Graphics()->QuadsDrawTL(&QuadItem, 1);
 			Graphics()->QuadsEnd();
 		}
@@ -429,7 +369,7 @@ void CSpectator::Spectate(int SpecMode, int SpectatorID)
 		return;
 	}
 
-	if(m_pClient->m_Snap.m_SpecInfo.m_SpecMode == SpecMode && m_pClient->m_Snap.m_SpecInfo.m_SpectatorID == SpectatorID)
+	if(m_pClient->m_Snap.m_SpecInfo.m_SpecMode == SpecMode && (SpecMode != SPEC_PLAYER || m_pClient->m_Snap.m_SpecInfo.m_SpectatorID == SpectatorID))
 		return;
 
 	CNetMsg_Cl_SetSpectatorMode Msg;
