@@ -7,17 +7,19 @@
 #include <engine/console.h>
 #include <engine/graphics.h>
 #include <engine/textrender.h>
+#include <engine/storage.h>
 
 #include <generated/client_data.h>
 #include <game/client/localization.h>
 #include <game/client/render.h>
 #include "editor.h"
 
+const char *pDefaultLayerName = "Tiles";
 
 CLayerTiles::CLayerTiles(int w, int h)
 {
 	m_Type = LAYERTYPE_TILES;
-	str_copy(m_aName, "Tiles", sizeof(m_aName));
+	str_copy(m_aName, pDefaultLayerName, sizeof(m_aName));
 	m_Width = w;
 	m_Height = h;
 	m_Image = -1;
@@ -36,6 +38,7 @@ CLayerTiles::CLayerTiles(int w, int h)
 	m_SaveTilesSize = 0;
 
 	m_SelectedRuleSet = 0;
+	m_LiveAutoMap = false;
 	m_SelectedAmount = 50;
 }
 
@@ -215,7 +218,7 @@ void CLayerTiles::Clamp(RECTi *pRect)
 
 void CLayerTiles::BrushSelecting(CUIRect Rect)
 {
-	vec4 FillColor = HexToRgba(g_Config.m_EdColorSelectionTile);
+	vec4 FillColor = HexToRgba(m_pEditor->Config()->m_EdColorSelectionTile);
 
 	Graphics()->TextureClear();
 	m_pEditor->Graphics()->QuadsBegin();
@@ -226,9 +229,10 @@ void CLayerTiles::BrushSelecting(CUIRect Rect)
 	m_pEditor->Graphics()->QuadsEnd();
 	char aBuf[16];
 	str_format(aBuf, sizeof(aBuf), "%d,%d", ConvertX(Rect.w), ConvertY(Rect.h));
-	TextRender()->Text(0, Rect.x+3.0f, Rect.y+3.0f, m_pEditor->m_ShowTilePicker?15.0f:15.0f*m_pEditor->m_WorldZoom, aBuf, -1);
+	TextRender()->Text(0, Rect.x+3.0f, Rect.y+3.0f, m_pEditor->m_ShowTilePicker?15.0f:15.0f*m_pEditor->m_WorldZoom, aBuf, -1.0f);
 }
 
+static int s_lastBrushX = -1, s_lastBrushY = -1;
 int CLayerTiles::BrushGrab(CLayerGroup *pBrush, CUIRect Rect)
 {
 	RECTi r;
@@ -251,6 +255,8 @@ int CLayerTiles::BrushGrab(CLayerGroup *pBrush, CUIRect Rect)
 		for(int x = 0; x < r.w; x++)
 			pGrabbed->m_pTiles[y*pGrabbed->m_Width+x] = m_pTiles[(r.y+y)*m_Width+(r.x+x)];
 
+	s_lastBrushX = -1;
+	s_lastBrushY = -1;
 	return 1;
 }
 
@@ -284,6 +290,13 @@ void CLayerTiles::FillSelection(bool Empty, CLayer *pBrush, CUIRect Rect)
 				m_pTiles[fy*m_Width+fx] = pLt->m_pTiles[(y*pLt->m_Width + x%pLt->m_Width) % (pLt->m_Width*pLt->m_Height)];
 		}
 	}
+
+	if(m_LiveAutoMap)
+	{
+		RECTi r = {sx - 1, sy - 1, w + 2, h + 2};	
+		m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper->Proceed(this, m_SelectedRuleSet, r);
+	}
+
 	m_pEditor->m_Map.m_Modified = true;
 }
 
@@ -292,11 +305,14 @@ void CLayerTiles::BrushDraw(CLayer *pBrush, float wx, float wy)
 	if(m_Readonly)
 		return;
 
-	//
 	CLayerTiles *l = (CLayerTiles *)pBrush;
 	int sx = ConvertX(wx);
 	int sy = ConvertY(wy);
 
+	//dont draw if the mouse is held without moving
+	if(sx == s_lastBrushX && sy == s_lastBrushY)
+		return;
+
 	for(int y = 0; y < l->m_Height; y++)
 		for(int x = 0; x < l->m_Width; x++)
 		{
@@ -307,7 +323,17 @@ void CLayerTiles::BrushDraw(CLayer *pBrush, float wx, float wy)
 
 			m_pTiles[fy*m_Width+fx] = l->m_pTiles[y*l->m_Width+x];
 		}
+
+	if(m_LiveAutoMap)
+	{
+		RECTi r = {sx - 1, sy - 1, l->m_Width + 2, l->m_Height + 2};
+		m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper->Proceed(this, m_SelectedRuleSet, r);
+	}
+
 	m_pEditor->m_Map.m_Modified = true;
+
+	s_lastBrushX = sx;
+	s_lastBrushY = sy;
 }
 
 void CLayerTiles::BrushFlipX()
@@ -324,6 +350,9 @@ void CLayerTiles::BrushFlipX()
 		for(int y = 0; y < m_Height; y++)
 			for(int x = 0; x < m_Width; x++)
 				m_pTiles[y*m_Width+x].m_Flags ^= m_pTiles[y*m_Width+x].m_Flags&TILEFLAG_ROTATE ? TILEFLAG_HFLIP : TILEFLAG_VFLIP;
+
+	s_lastBrushX = -1;
+	s_lastBrushY = -1;
 }
 
 void CLayerTiles::BrushFlipY()
@@ -340,6 +369,9 @@ void CLayerTiles::BrushFlipY()
 		for(int y = 0; y < m_Height; y++)
 			for(int x = 0; x < m_Width; x++)
 				m_pTiles[y*m_Width+x].m_Flags ^= m_pTiles[y*m_Width+x].m_Flags&TILEFLAG_ROTATE ? TILEFLAG_VFLIP : TILEFLAG_HFLIP;
+
+	s_lastBrushX = -1;
+	s_lastBrushY = -1;
 }
 
 void CLayerTiles::BrushRotate(float Amount)
@@ -377,6 +409,9 @@ void CLayerTiles::BrushRotate(float Amount)
 		BrushFlipX();
 		BrushFlipY();
 	}
+
+	s_lastBrushX = -1;
+	s_lastBrushY = -1;
 }
 
 void CLayerTiles::Resize(int NewW, int NewH)
@@ -427,13 +462,17 @@ void CLayerTiles::Shift(int Direction)
 				mem_copy(&m_pTiles[y*m_Width], &m_pTiles[(y-1)*m_Width], m_Width*sizeof(CTile));
 		}
 	}
+
+	s_lastBrushX = -1;
+	s_lastBrushY = -1;
 }
 
 void CLayerTiles::ShowInfo()
 {
 	float ScreenX0, ScreenY0, ScreenX1, ScreenY1;
 	Graphics()->GetScreen(&ScreenX0, &ScreenY0, &ScreenX1, &ScreenY1);
-	Graphics()->TextureSet(m_pEditor->Client()->GetDebugFont());
+	static IGraphics::CTextureHandle s_Font = Graphics()->LoadTexture("ui/debug_font.png", IStorage::TYPE_ALL, CImageInfo::FORMAT_AUTO, IGraphics::TEXLOAD_NORESAMPLE);
+	Graphics()->TextureSet(s_Font);
 	Graphics()->QuadsBegin();
 
 	int StartY = max(0, (int)(ScreenY0/32.0f)-1);
@@ -447,7 +486,7 @@ void CLayerTiles::ShowInfo()
 			int c = x + y*m_Width;
 			if(m_pTiles[c].m_Index)
 			{
-				char aBuf[64];
+				char aBuf[32];
 				str_format(aBuf, sizeof(aBuf), "%i", m_pTiles[c].m_Index);
 				m_pEditor->Graphics()->QuadsText(x*32, y*32, 16.0f, aBuf);
 
@@ -456,6 +495,14 @@ void CLayerTiles::ShowInfo()
 									m_pTiles[c].m_Flags&TILEFLAG_ROTATE? 'R' : ' ',
 									0};
 				m_pEditor->Graphics()->QuadsText(x*32, y*32+16, 16.0f, aFlags);
+
+				// TODO: Use text render instead, once it's optimized enough for this much text at once, and remove usage of debug font here
+				/*str_format(aBuf, sizeof(aBuf),
+					"%i\n%c%c%c", m_pTiles[c].m_Index,
+						m_pTiles[c].m_Flags&TILEFLAG_VFLIP ? 'V' : ' ',
+						m_pTiles[c].m_Flags&TILEFLAG_HFLIP ? 'H' : ' ',
+						m_pTiles[c].m_Flags&TILEFLAG_ROTATE? 'R' : ' ');
+				TextRender()->Text(0, x*32+4, y*32+4, 10.0f, aBuf, 32.0f);*/
 			}
 			x += m_pTiles[c].m_Skip;
 		}
@@ -468,8 +515,9 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 {
 	CUIRect Button;
 
+	bool IsGameLayer = m_pEditor->m_Map.m_pGameLayer == this;
 	bool InGameGroup = !find_linear(m_pEditor->m_Map.m_pGameGroup->m_lLayers.all(), this).empty();
-	if(m_pEditor->m_Map.m_pGameLayer != this)
+	if(!IsGameLayer)
 	{
 		if(m_Image >= 0 && m_Image < m_pEditor->m_Map.m_lImages.size() && m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper)
 		{
@@ -483,7 +531,8 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 			{
 				if(m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper->GetType() == IAutoMapper::TYPE_TILESET)
 				{
-					m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper->Proceed(this, m_SelectedRuleSet);
+					RECTi r = {0, 0, m_Width, m_Height};
+					m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper->Proceed(this, m_SelectedRuleSet, r);
 					return 1; // only close the popup when it's a tileset
 				}
 				else if(m_pEditor->m_Map.m_lImages[m_Image]->m_pAutoMapper->GetType() == IAutoMapper::TYPE_DOODADS)
@@ -546,7 +595,7 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 		{0},
 	};
 
-	if(m_pEditor->m_Map.m_pGameLayer == this) // remove the image and color properties if this is the game layer
+	if(IsGameLayer) // remove the image and color properties if this is the game layer
 	{
 		aProps[3].m_pName = 0;
 		aProps[4].m_pName = 0;
@@ -572,7 +621,14 @@ int CLayerTiles::RenderProperties(CUIRect *pToolBox)
 			m_Image = -1;
 		}
 		else
+		{
+			bool HasNameOfOldImage = m_Image != -1 && str_comp(m_aName, m_pEditor->m_Map.m_lImages[m_Image]->m_aName) == 0;
 			m_Image = NewVal%m_pEditor->m_Map.m_lImages.size();
+			m_SelectedRuleSet = 0;
+			m_LiveAutoMap = false;
+			if(str_comp(m_aName, pDefaultLayerName) == 0 || HasNameOfOldImage)
+				str_copy(m_aName, m_pEditor->m_Map.m_lImages[m_Image]->m_aName, sizeof(m_aName));
+		}
 	}
 	else if(Prop == PROP_COLOR)
 	{
