@@ -12,7 +12,7 @@ CFlag::CFlag(CGameWorld *pGameWorld, int Team, vec2 StandPos)
 	m_Team = Team;
 	m_StandPos = StandPos;
 
-	GameServer()->m_World.InsertEntity(this);
+	GameWorld()->InsertEntity(this);
 
 	Reset();
 }
@@ -67,7 +67,7 @@ void CFlag::TickDefered()
 			}
 			else
 			{
-				m_Vel.y += GameServer()->m_World.m_Core.m_Tuning.m_Gravity;
+				m_Vel.y += GameWorld()->m_Core.m_Tuning.m_Gravity;
 				GameServer()->Collision()->MoveBox(&m_Pos, &m_Vel, vec2(ms_PhysSize, ms_PhysSize), 0.5f);
 			}
 		}
