@@ -3,6 +3,12 @@
 #ifndef GAME_SERVER_ENTITIES_PROJECTILE_H
 #define GAME_SERVER_ENTITIES_PROJECTILE_H
 
+enum
+{
+	PLAYER_TEAM_BLUE = -2,
+	PLAYER_TEAM_RED = -1
+};
+
 class CProjectile : public CEntity
 {
 public:
@@ -12,6 +18,9 @@ public:
 	vec2 GetPos(float Time);
 	void FillInfo(CNetObj_Projectile *pProj);
 
+	int GetOwner() const { return m_Owner; }
+	void LoseOwner();
+
 	virtual void Reset();
 	virtual void Tick();
 	virtual void TickPaused();
@@ -21,6 +30,7 @@ private:
 	vec2 m_Direction;
 	int m_LifeSpan;
 	int m_Owner;
+	int m_OwnerTeam;
 	int m_Type;
 	int m_Damage;
 	int m_SoundImpact;
