@@ -6,6 +6,7 @@
 #include <engine/console.h>
 #include <engine/server.h>
 
+#include <game/commands.h>
 #include <game/layers.h>
 #include <game/voting.h>
 
@@ -36,6 +37,7 @@
 class CGameContext : public IGameServer
 {
 	IServer *m_pServer;
+	class CConfig *m_pConfig;
 	class IConsole *m_pConsole;
 	CLayers m_Layers;
 	CCollision m_Collision;
@@ -44,7 +46,7 @@ class CGameContext : public IGameServer
 
 	static void ConTuneParam(IConsole::IResult *pResult, void *pUserData);
 	static void ConTuneReset(IConsole::IResult *pResult, void *pUserData);
-	static void ConTuneDump(IConsole::IResult *pResult, void *pUserData);
+	static void ConTunes(IConsole::IResult *pResult, void *pUserData);
 	static void ConPause(IConsole::IResult *pResult, void *pUserData);
 	static void ConChangeMap(IConsole::IResult *pResult, void *pUserData);
 	static void ConRestart(IConsole::IResult *pResult, void *pUserData);
@@ -64,12 +66,16 @@ class CGameContext : public IGameServer
 	static void ConchainSettingUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainGameinfoUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 
+	static void NewCommandHook(const CCommandManager::CCommand *pCommand, void *pContext);
+	static void RemoveCommandHook(const CCommandManager::CCommand *pCommand, void *pContext);
+
 	CGameContext(int Resetting);
 	void Construct(int Resetting);
 
 	bool m_Resetting;
 public:
 	IServer *Server() const { return m_pServer; }
+	class CConfig *Config() { return m_pConfig; }
 	class IConsole *Console() { return m_pConsole; }
 	CCollision *Collision() { return &m_Collision; }
 	CTuningParams *Tuning() { return &m_Tuning; }
@@ -84,6 +90,9 @@ public:
 
 	class IGameController *m_pController;
 	CGameWorld m_World;
+	CCommandManager m_CommandManager;
+
+	CCommandManager *CommandManager() { return &m_CommandManager; }
 
 	// helper functions
 	class CCharacter *GetPlayerChar(int ClientID);
@@ -119,6 +128,9 @@ public:
 
 		VOTE_TIME=25,
 		VOTE_CANCEL_TIME = 10,
+
+		MIN_SKINCHANGE_CLIENTVERSION = 0x0703,
+		MIN_RACE_CLIENTVERSION = 0x0704,
 	};
 	class CHeap *m_pVoteOptionHeap;
 	CVoteOptionServer *m_pVoteOptionFirst;
@@ -139,11 +151,16 @@ public:
 	void SendWeaponPickup(int ClientID, int Weapon);
 	void SendMotd(int ClientID);
 	void SendSettings(int ClientID);
+	void SendSkinChange(int ClientID, int TargetID);
 
 	void SendGameMsg(int GameMsgID, int ClientID);
 	void SendGameMsg(int GameMsgID, int ParaI1, int ClientID);
 	void SendGameMsg(int GameMsgID, int ParaI1, int ParaI2, int ParaI3, int ClientID);
 
+	void SendChatCommand(const CCommandManager::CCommand *pCommand, int ClientID);
+	void SendChatCommands(int ClientID);
+	void SendRemoveChatCommand(const CCommandManager::CCommand *pCommand, int ClientID);
+
 	//
 	void CheckPureTuning();
 	void SendTuningParams(int ClientID);
@@ -163,24 +180,28 @@ public:
 
 	virtual void OnMessage(int MsgID, CUnpacker *pUnpacker, int ClientID);
 
-	virtual void OnClientConnected(int ClientID) { OnClientConnected(ClientID, false); }
-	void OnClientConnected(int ClientID, bool Dummy);
+	virtual void OnClientConnected(int ClientID, bool AsSpec) { OnClientConnected(ClientID, false, AsSpec); }
+	void OnClientConnected(int ClientID, bool Dummy, bool AsSpec);
 	void OnClientTeamChange(int ClientID);
 	virtual void OnClientEnter(int ClientID);
 	virtual void OnClientDrop(int ClientID, const char *pReason);
 	virtual void OnClientDirectInput(int ClientID, void *pInput);
 	virtual void OnClientPredictedInput(int ClientID, void *pInput);
 
+	virtual bool IsClientBot(int ClientID) const;
 	virtual bool IsClientReady(int ClientID) const;
 	virtual bool IsClientPlayer(int ClientID) const;
+	virtual bool IsClientSpectator(int ClientID) const;
 
 	virtual const char *GameType() const;
 	virtual const char *Version() const;
 	virtual const char *NetVersion() const;
+	virtual const char *NetVersionHashUsed() const;
+	virtual const char *NetVersionHashReal() const;
 };
 
 inline int64 CmaskAll() { return -1; }
-inline int64 CmaskOne(int ClientID) { return 1<<ClientID; }
+inline int64 CmaskOne(int ClientID) { return (int64)1<<ClientID; }
 inline int64 CmaskAllExceptOne(int ClientID) { return CmaskAll()^CmaskOne(ClientID); }
 inline bool CmaskIsSet(int64 Mask, int ClientID) { return (Mask&CmaskOne(ClientID)) != 0; }
 #endif
