@@ -13,6 +13,20 @@ CGameControllerLTS::CGameControllerLTS(CGameContext *pGameServer) : IGameControl
 	m_GameFlags = GAMEFLAG_TEAMS|GAMEFLAG_SURVIVAL;
 }
 
+// event
+void CGameControllerLTS::OnCharacterSpawn(class CCharacter *pChr)
+{
+	IGameController::OnCharacterSpawn(pChr);
+
+	// give start equipment
+	pChr->GiveWeapon(WEAPON_SHOTGUN, 10);
+	pChr->GiveWeapon(WEAPON_GRENADE, 10);
+	pChr->GiveWeapon(WEAPON_LASER, 5);
+
+	// prevent respawn
+	pChr->GetPlayer()->m_RespawnDisabled = GetStartRespawnState();
+}
+
 // game
 void CGameControllerLTS::DoWincheckRound()
 {
@@ -20,7 +34,7 @@ void CGameControllerLTS::DoWincheckRound()
 	for(int i = 0; i < MAX_CLIENTS; ++i)
 	{
 		if(GameServer()->m_apPlayers[i] && GameServer()->m_apPlayers[i]->GetTeam() != TEAM_SPECTATORS &&
-			(!GameServer()->m_apPlayers[i]->m_RespawnDisabled || 
+			(!GameServer()->m_apPlayers[i]->m_RespawnDisabled ||
 			(GameServer()->m_apPlayers[i]->GetCharacter() && GameServer()->m_apPlayers[i]->GetCharacter()->IsAlive())))
 			++Count[GameServer()->m_apPlayers[i]->GetTeam()];
 	}
